/**
 * Progresser for Elementor
 * Customizable progress bars for Elementor editor.
 * Exclusively on https://1.envato.market/progresser-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

"use strict";

const mdpProgresserElementor = {
    getBarOptions: function ( $box ) {
        return {
            maxCounterValue: $box.dataset.maxCounterValue,
            counterValue: $box.dataset.counterValue,
            progressBarSize: $box.dataset.progressBarHeight,
            innerTextAlignment: $box.dataset.innerTextAlignment,
            innerText: $box.dataset.innerText,
            animationDuration: $box.dataset.animationDuration,
            animationEasing: $box.dataset.animationEasing,
            startAnimation: $box.dataset.startAnimation
        }
    },

    createCircleTextElement: function ( innerText, showCounterValue, symbolPosition, counterValue, symbol, reflectTextClass ) {

        if ( showCounterValue !== 'yes' && !innerText ) { return ''; }

        const wrapper = document.createElement( 'span' );
        const innerTextWrapper = document.createElement( 'span' );
        const countValueWrapper = document.createElement( 'span' );
        const countValueNumber = document.createElement( 'span' );
        const countSymbol = document.createElement( 'span' );

        wrapper.classList.add( 'mdp-progresser-elementor-inner-text' );

        if ( reflectTextClass ) {
            wrapper.classList.add( reflectTextClass );
        }

        /** Add inner text */
        if ( innerText ) {
            innerTextWrapper.classList.add( 'mdp-progresser-elementor-inner-text-value' );
            innerTextWrapper.innerText = innerText;
            wrapper.appendChild( innerTextWrapper );
        }

        /** Add counter value */
        if ( showCounterValue === 'yes' ) {
            countValueWrapper.classList.add( 'mdp-progress-elementor-count-value', `mdp-progress-count-symbol-position-${symbolPosition}` );
            countValueNumber.classList.add( 'mdp-progress-elementor-count-number' );
            countValueNumber.innerText = counterValue;
            countSymbol.classList.add( 'mdp-progress-elementor-count-symbol' );
            countSymbol.innerText = symbol;
            countValueWrapper.appendChild( countValueNumber );
            countValueWrapper.appendChild( countSymbol );
            wrapper.appendChild( countValueWrapper );
        }

        return wrapper;
    },

    getProgressBarAnimation: function ( bar, barOptions, container ) {
        const opts = {
            duration: +barOptions.animationDuration,
            easing: barOptions.animationEasing,
        };

        if ( barOptions.startAnimation === 'in-viewport' ) {
            let animated = true;
            function checkInViewPort() {
                if ( container.getBoundingClientRect().bottom <= ( window.innerHeight || document.documentElement.clientHeight ) && container.getBoundingClientRect().top > 0 ) {
                    if ( animated ) {
                        animated = false;
                        bar.animate( barOptions.counterValue/barOptions.maxCounterValue, opts, () => { animated = true } );
                    }
                } else {
                    bar.set( 0 );
                }
            }
            checkInViewPort()
            window.addEventListener( 'scroll', () => {
                checkInViewPort();
            } );
        } else if ( barOptions.startAnimation === 'one-time' ) {
            bar.animate( barOptions.counterValue/barOptions.maxCounterValue, opts );
        } else {
            let animated = true;
            let lastScroll = 0;
            window.addEventListener( 'scroll', () => {
                let currentScroll = document.documentElement.scrollTop || document.body.scrollTop; // Get Current Scroll Value

                // animate only on scroll down
                if ( currentScroll > 0 && lastScroll <= currentScroll ){
                    lastScroll = currentScroll;
                    if ( animated ) {
                        animated = false;
                        bar.set( 0 );
                        bar.animate( barOptions.counterValue/barOptions.maxCounterValue, opts, () => { animated = true } );
                    }
                }
            } );
        }
    },

    getCircleTypeOptions: function ( $box, widgetId ) {
        const container = document.querySelector( `.mdp-progresser-bar-container-${widgetId}` );
        const barOptions = this.getBarOptions( $box );
        const reflectCounterText = $box.dataset.reflectCounterText;
        const showCounterValue = $box.dataset.showCounterValue;
        const symbolPosition = $box.dataset.symbolPosition;
        const postfixPrefixSymbol = $box.dataset.postfixPrefixSymbol;

        let valueBlock = '';

        setTimeout( () => {
            valueBlock = $box.querySelector( '.mdp-progress-elementor-count-number' );
        }, 0 );


        const reflectTextClass = reflectCounterText === 'yes' ? 'mdp-progresser-elementor-inner-text-reflect' : '';

        /** Inner text and counter element */
        const textElement = this.createCircleTextElement(
            barOptions.innerText,
            showCounterValue,
            symbolPosition,
            barOptions.counterValue,
            postfixPrefixSymbol,
            reflectTextClass
        );

        return {
            container,
            barOptions,
            reflectTextClass,
            showCounterValue,
            symbolPosition,
            postfixPrefixSymbol,
            valueBlock,
            textElement

        };
    },

    lineProgressBar: function ( widgetId ) {
        const $box = document.querySelector( `.mdp-progresser-line-bar-elementor-box-${widgetId}` );
        const valueBlock = $box.querySelector( '.mdp-progress-elementor-count-number' );
        const container = document.querySelector( `.mdp-progresser-bar-container-${widgetId}` );
        const barOptions = this.getBarOptions( $box )


        const line = new ProgressBar.Line( `.mdp-progresser-bar-container-${widgetId}`, {
            strokeWidth: barOptions.progressBarSize,
            trailWidth: barOptions.progressBarSize,
            text: {
                className: `mdp-progresser-elementor-inner-text mdp-progresser-elementor-inner-text-position-${barOptions.innerTextAlignment} ?>`,
                    value: barOptions.innerText,
                    style: {
                    position: 'absolute',
                        top: '50%',
                        transform: {
                        prefix: true,
                            value: 'translateY(-50%)'
                    }
                }
            },
            step: ( state, line ) => {
                if ( valueBlock ) {
                    valueBlock.textContent = Math.round( line.value() * barOptions.maxCounterValue ) + '';
                }
            },
        } );


        this.getProgressBarAnimation( line, barOptions, container );

    },

    circleProgressBar: function ( widgetId ) {
        const $box = document.querySelector( `.mdp-progresser-circle-bar-elementor-box-${widgetId}` );
        const circleOptions = this.getCircleTypeOptions( $box, widgetId );

        const circle = new ProgressBar.Circle( `.mdp-progresser-bar-container-${widgetId}`, {
            strokeWidth: circleOptions.barOptions.progressBarSize,
            trailWidth: circleOptions.barOptions.progressBarSize,
            text: {
                className: `mdp-progresser-elementor-inner-text-wrapper`,
                    value: circleOptions.textElement,
                    style: {
                    position: 'absolute',
                        top: '50%',
                        transform: {
                        prefix: true,
                            value: 'translateY(-50%)'
                    }
                }
            },
            step: ( state, circle ) => {
                if ( circleOptions.valueBlock ) {
                    circleOptions.valueBlock.textContent = Math.round( circle.value() * circleOptions.barOptions.maxCounterValue ) + '';
                }
            },
        } );

        this.getProgressBarAnimation( circle, circleOptions.barOptions, circleOptions.container );

    },

    semiCircleProgressBar: function ( widgetId ) {
        const $box = document.querySelector( `.mdp-progresser-semicircle-bar-elementor-box-${widgetId}` );
        const semicircleOptions = this.getCircleTypeOptions( $box, widgetId );


        const semicircle = new ProgressBar.SemiCircle( `.mdp-progresser-bar-container-${widgetId}`, {
            strokeWidth: semicircleOptions.barOptions.progressBarSize,
            trailWidth: semicircleOptions.barOptions.progressBarSize,
            text: {
                className: `mdp-progresser-elementor-inner-text-wrapper`,
                value: semicircleOptions.textElement
            },
            step: ( state, semicircle ) => {
                if ( semicircleOptions.valueBlock ) {
                   semicircleOptions.valueBlock.textContent = Math.round( semicircle.value() * semicircleOptions.barOptions.maxCounterValue ) + '';
                }
            },
        } );

        this.getProgressBarAnimation( semicircle, semicircleOptions.barOptions, semicircleOptions.container );

    }
};



jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-progresser-line-bar' ) ) {
            mdpProgresserElementor.lineProgressBar.call( mdpProgresserElementor, $scope[0].dataset.id );
        } else if ( $scope[0].classList.contains( 'elementor-widget-mdp-progresser-circle-bar' ) ) {
            mdpProgresserElementor.circleProgressBar.call( mdpProgresserElementor, $scope[0].dataset.id );
        } else if ( $scope[0].classList.contains( 'elementor-widget-mdp-progresser-semicircle-bar' ) ) {
            mdpProgresserElementor.semiCircleProgressBar.call( mdpProgresserElementor, $scope[0].dataset.id );
        }
    } );

} );