<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use Elementor\Icons_Manager;
use Merkulove\Selection\Unity\Plugin;
use Merkulove\Selection\Unity\Settings;
use Merkulove\Selection\Unity\TabActivation;
use Merkulove\Selection\Unity\UI;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

	/**
	 * Array of duplicated activated plugins that used Selection
	 * @var array
	 */
	private static $duplicates = array();

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        $this->both_hooks();

        $this->public_hooks();

        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */

        /** Add Sticky Effect */
        StickyEffect::get_instance();

        /** Add Scrolling Effect */
        ScrollingEffect::get_instance();

	    /** Add Elementor Widgets */
	    Elementor::get_instance()->register_elementor_widgets();

	    /** Register Selection Header Post Type */
	    TemplatesMetaboxes::get_instance();

        /** Templates Library */
	    TemplatesLibrary::get_instance()->setup();

        /** Async Search For Search Bar */
        add_action( 'wp_ajax_mdp_async_search', [ $this, 'get_search_results' ] );
        add_action( 'wp_ajax_nopriv_mdp_async_search', [ $this, 'get_search_results' ] );

    }

    /**
     * Register all the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Selection Templates */
        Templates::get_instance();

	    /** Shortcode */
	    Shortcode::get_instance();

        /** Add Schema markup. */
        add_action( 'wp_head', [$this, 'structured_data'] );
        add_action( 'wp_head', [$this, 'structured_data_breadcrumbs'] );

        /** Selection Global Custom CSS */
	    add_action( 'wp_enqueue_scripts', [$this, 'add_global_custom_css'] );

        /** Search filter for search bar */
        add_action( 'pre_get_posts', [$this, 'search_filter'], 9 );

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

	    /**  Add custom category. */
	    add_action( 'elementor/elements/categories_registered', [ $this, 'add_elementor_widget_categories' ] );

	    /** Add meta-box scripts and styles */
	    add_action( 'current_screen', [ $this, 'enqueue_edit' ] );

	    /** Show message on the plugins page */
	    add_action( 'current_screen', [ $this, 'active_merkulove_plugins' ] );

	    /** Render notices */
	    add_action( 'admin_notices', [ $this, 'render_notice_duplicate' ] );
	    add_action( 'admin_footer', [ $this, 'render_snackbar_duplicate' ] );

	    /** Show activation warning */
	    add_action( 'admin_footer', [ $this, 'not_activated_notice' ] );

	    /** Templates library */
	    TemplateShortcode::get_instance();

        /** Register WooCommerce frontend */
        if ( $this->is_active_woocommerce() ) {
            add_action( 'init', [$this, 'register_wc_frontend'], 5 );
        }

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {}

	/**
     * Add inline custom CSS to elementor frontend styles handler
	 * @return void
	 */
    public function add_global_custom_css() {

        $custom_css = Settings::get_instance()->options[ 'custom_css' ];

        if ( $custom_css ) {

	        wp_add_inline_style( 'elementor-frontend', $custom_css );

        }

    }

    /**
     * Create title icon.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_widgeter_title_icon( $title_icon, $enabled ) {

        if ( empty( $title_icon ) || !$enabled ) { return ''; }

        $icon = $title_icon['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $title_icon['value'] ) :
            '<i class="'.esc_attr( $title_icon['value'] ).'"></i>';

        return sprintf(
            '<div class="mdp-widgeter-elementor-title-icon">%s</div>',
            $icon
        );
    }

    public function widgeter_get_widget_default_args( $type, $title_tag, $icon_enabled, $icon, $icon_position, $align_class = '', $before_widget = '' ) {
        $disable_default_icon = $type === 'rss-feed' && $icon_enabled ? 'mdp-widgeter-disable-default-icon' : '';

        $default_args = [
            'before_widget' => '<div 
                                class="mdp-widgeter-elementor-box  
                                mdp-widgeter-'.$type.'-elementor-box '.$align_class . ' '. $disable_default_icon . ' "
                                >',
            'after_widget' => '</div>',
            'before_title' => sprintf(
                '<div class="mdp-widgeter-elementor-title-wrapper"> 
                                   <%s class="
                                        mdp-widgeter-title 
                                        mdp-widgeter-%s-title 
                                        mdp-widgeter-elementor-icon-position-%s">%s',
                $title_tag,
                $type,
                esc_attr( $icon_position ),
                $this->create_widgeter_title_icon( $icon, $icon_enabled )
            ),
            'after_title' => sprintf( '</%s></div>', $title_tag )
        ];

        /** Set custom before widget */
        if ( !empty( $before_widget ) ) { $default_args['before_widget'] = $before_widget; }

        return $default_args;
    }


    /**
     * Hide product parts actions.
     *
     * @param $settings
     * @param $args
     * @return void
     * @since 1.0.0
     * @access private
     */
    public function remove_product_actions( $settings, $args ) {
        if ( $settings[$args['show_cart_key']] !== 'yes' ) {
            remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart');
            remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart');
        }

        if ( $settings[$args['show_price_key']] !== 'yes' ) {
            remove_action(
                'woocommerce_after_shop_loop_item_title',
                'woocommerce_template_loop_price',
                10
            );
        }
    }


    /**
     * Restore defaults actions.
     *
     * @param $settings
     * @param $args
     * @return void
     * @since 1.0.0
     * @access private
     */
    public function restore_product_actions( $settings, $args ) {
        if ( $settings[$args['show_cart_key']] !== 'yes' ) {
            add_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart');
            add_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart');
        }

        if ( $settings[$args['show_price_key']] !== 'yes' ) {
            add_action(
                'woocommerce_after_shop_loop_item_title',
                'woocommerce_template_loop_price',
                10
            );
        }
    }

    /**
     * Filters search by post type.
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function search_filter( $query ) {
        if ( $query->is_search() ) {
            if ( isset( $_GET['mdp_post_type'] ) ) {
                $post_type = array_map( 'sanitize_text_field', $_GET['mdp_post_type'] );
                $query->set( 'post_type', $post_type );
            }
        }

        return $query;
    }

    /**
     * Find form widget method.
     *
     * @param $elements
     * @param $form_id
     * @return bool
     * @since 1.0.0
     * @access public
     *
     */
    private function find_form( $elements, $form_id ) {
        foreach ( $elements as $element ) {
            if ( $form_id === $element['id'] ) {
                return $element;
            }

            if ( ! empty( $element['elements'] ) ) {
                $element = self::find_form( $element['elements'], $form_id );

                if ( $element ) {
                    return $element;
                }
            }
        }

        return false;
    }

    /**
     * This method returns search results
     *
     * @since 1.0.0
     * @access public
     */
    public function get_search_results() {
        if ( empty( $_GET['mdp_searcher_nonce'] ) ||
            ! wp_verify_nonce( $_GET['mdp_searcher_nonce'], 'mdp-searcher-elementor-nonce' ) ) { return; }

        $search_result = [];

        $search_value = sanitize_text_field( $_GET['mdp_search_value'] );

        $post_types = explode( ',', sanitize_text_field( $_GET['mdp_post_types'] ) );

        $results_quantity = sanitize_text_field( $_GET['mdp_results_quantity'] );

        $query = new \WP_Query( [
            'posts_per_page' => $results_quantity,
            's' => $search_value,
            'post_type' => $post_types,
            'post_status' => 'publish'
        ] );

        $plugin_elementor = \Elementor\Plugin::instance();

        $form_id = isset( $_GET['mdp_form_id'] ) ? sanitize_text_field( $_GET['mdp_form_id'] ) : null;
        $post_id = isset( $_GET['mdp_post_id'] ) ? sanitize_text_field( $_GET['mdp_post_id'] ) : null;

        $document = $plugin_elementor->documents->get( $post_id );

        $form = null;

        if ( !empty( $document ) ) {
            $form = $this->find_form( $document->get_elements_data(), $form_id );
        }

        $form_settings = $form['settings'];

        $empty_result_text = empty( $form_settings['empty_result_text'] ) ?
            'Empty search result!' : esc_html( $form_settings['empty_result_text'] );

        if ( $query->have_posts() ) {
            while ( $query->have_posts() ) {
                $query->the_post();
                $post_image = sprintf(
                    '<div class="mdp-searcher-elementor-search-img-container"><img src="%s" alt=""></div>',
                    esc_url( get_the_post_thumbnail_url() )
                );
                $search_result[] = sprintf(
                    "<div class='mdp-searcher-elementor-result-block'>
                                <a class='mdp-searcher-elementor-search-result' href='%s'>
                                    %s
                                    <div class='mdp-searcher-elementor-search-content'>
                                        <div class='mdp-searcher-elementor-search-post-title'>%s</div>
                                        <div class='mdp-searcher-elementor-search-post-excerpt'>%s</div>
                                    </div>
                                </a>
                            </div>",
                    esc_url( get_permalink() ),
                    !empty( get_the_post_thumbnail_url() ) ?
                        $post_image :
                        '',
                    the_title( '', '', false ),
                    strlen( get_the_excerpt() ) > 20 ?
                        substr( esc_html( get_the_excerpt() ), 0, 20 ) . '...' :
                        get_the_excerpt()
                );
            }
            wp_send_json_success( $search_result );
        } else {
            wp_send_json_success( [ 'empty_text' => $empty_result_text, 'is_empty' => true ] );
        }

    }

	/**
	 * Find and store activated merkulove plugins for Elementor
	 */
	public function active_merkulove_plugins() {

		$screen = get_current_screen();
		if ( null === $screen ) { return; }

        // Run only on the Plugins page and Selection settings
		if ( ! in_array( $screen->id, [ 'plugins', 'toplevel_page_mdp_selection_settings' ] ) ) { return; }

		// Get active plugins
		$active_plugin = get_option('active_plugins');
		if ( ! is_array( $active_plugin ) ) { return; }

		// Get selection plugins
		$selection_plugins = get_option( 'mdp_selection_widgets_settings' );
		if ( ! is_array( $selection_plugins ) ) { return; }

		foreach ( $active_plugin as $plugin_path ) {

			$plugin_slug = explode( '/', $plugin_path )[ 0 ];

			// Find plugins contains -elementor in the file name
			if ( strpos( $plugin_slug, '-elementor' ) ) {

				$selection_slug = str_replace( '-elementor', '', $plugin_slug );

				if ( isset( $selection_plugins[ $selection_slug ] ) && $selection_plugins[ $selection_slug ] === 'on' ) {

					array_push( self::$duplicates, ucfirst( $selection_slug ) );

				}

			}

		}

	}

	/**
	 * Render message about duplicated elementor widgets as single plugin by merkulove
	 */
	public function render_notice_duplicate() {

	    if ( empty( self::$duplicates ) ) { return; }

	    $duplicates = preg_filter('/$/', ' for Elementor', self::$duplicates );
		?>
		<div class="notice notice-warning is-dismissible">
			<p>
				<strong>Selection</strong><?php esc_html_e( ' already uses the functionality of these plugins: ', 'selection' ); ?>
                <strong><?php echo implode( ', ', $duplicates ); ?></strong>.
				<?php esc_html_e( ' You can safely deactivate these plugins.', 'selection' ); ?>
			</p>
		</div>
		<?php

	}

    /**
     * Output ld+json Markup.
     *
     * @since 1.0.0
     * @access public
     * @return void
     **/
    public function structured_data() {

        $data = $this->get_data_for_schema();

        /** Exit on the Category and Tag pages */
        if ( is_category() || is_tag() ) { return; }

        if ( count( $data ) > 0 ) {

            $title = htmlentities( get_post_field( 'post_title' ) );
            $best = 5;
            $vote_count = get_transient( 'vote_count' ) === false ? 1 : get_transient( 'vote_count' ) + 0.1;
            set_transient( 'vote_count', $vote_count );
            $count = floor( $vote_count );
            $min = floatval( $data['rating_count'] ) - 0.1;
            $max = floatval( $data['rating_count'] ) - 0.01;
            $value = mt_rand( $min * 100, $max * 100 ) / 100;
            $author = $data['author_name'];
            $description = wp_strip_all_tags( $data['testimonial_description'] );

            if ( $data['rating_count'] === '' || $data['enable_rich_snippet'] === 'no' ) {
                return;
            }

            $json_ld = $data['json_schema_markup'];

            $json_ld = str_replace( ['[title]', '[best]', '[value]', '[count]', '[author]', '[description]'], [$title, $best, $value, $count, $author, $description], $json_ld );

            printf( '<script type="application/ld+json">%s</script>', $json_ld );
        }

    }

    /**
     * Get data for ld+json Markup.
     *
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_data_for_schema() {

        global $wpdb;

        $result = [];

        $post_id = get_the_ID();

        $data = $wpdb->get_row(
            $wpdb->prepare( "SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE `meta_key` = '_elementor_data'  AND post_id = %d", $post_id )
        );

        if ( !empty( $data ) ) {

            $meta_value = json_decode( $data->meta_value );
            $plugin_id = get_transient( 'plugin_id' );

            foreach ( $meta_value as $item ) {
                foreach ( $item->elements as $element ) {
                    foreach ( $element->elements as $sub_element ) {
                        if ( $sub_element->id === $plugin_id ) {
                            $result['author_name'] = isset( $sub_element->settings->author_name ) ? $sub_element->settings->author_name : '';
                            $result['rating_count'] = isset( $sub_element->settings->rating_count ) ? $sub_element->settings->rating_count : '';
                            $result['testimonial_description'] = isset( $sub_element->settings->testimonial_description ) ? $sub_element->settings->testimonial_description : '';
                            $result['json_schema_markup'] = $sub_element->settings->json_schema_markup;
                            $result['enable_rich_snippet'] = isset( $sub_element->settings->enable_rich_snippet ) && $sub_element->settings->enable_rich_snippet === 'yes' ? 'yes' : 'no';
                        }
                    }
                }
            }
        }

        return $result;

    }

    /**
     * Clears html tags from archive title.
     *
     * @return string
     * @since 1.0.0
     * @access public
     *
     */
    public function archive_title_filter( $title ) {
        return strip_tags( $title );
    }

    /**
     * Generates data for breadcrumbs.
     *
     * @return array
     * @since 1.0.0
     * @access public
     *
     */
    public function get_data_breadcrumbs( $parent_pages_level ) {

        /** Add filters for archive title */
        add_filter( 'get_the_archive_title', [ $this, 'archive_title_filter' ] );
        add_filter( 'get_the_archive_title_prefix', '__return_false' );

        $breadcrumbs = [];

        // check if woocommerce plugin is active
        $woocommerce_active = false;
        if (  in_array('woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
            $woocommerce_active = true;
        }

        $page_num = ( get_query_var( 'paged' ) ) ? get_query_var( 'paged' ) : 1;

        if ( is_front_page() ) {
            if ( $page_num > 1 ) {
                $breadcrumbs[ 'homepage' ] = [ 'name' => get_bloginfo( 'name' ), 'is_homepage' => true, 'is_parent' => false, 'is_child' => false, 'link' => site_url(), 'is_active' => true, 'element' => '',  ];

            }
        } else {
            if ( $woocommerce_active ) {
                if ( !is_woocommerce() ) {
                    $breadcrumbs[ 'homepage' ] = [ 'name' => get_bloginfo( 'name' ), 'is_homepage' => true, 'is_parent' => false, 'is_child' => false, 'link' => site_url(), 'is_active' => false, 'element' => '' ];
                }
            } else {
                $breadcrumbs[ 'homepage' ] = [ 'name' => get_bloginfo( 'name' ), 'is_homepage' => true, 'is_parent' => false, 'is_child' => false, 'link' => site_url(), 'is_active' => false, 'element' => '' ];
            }
            if ( get_post_type() === 'post' && !is_archive() ) {
                $post_categories = get_the_category();
                if ( !empty( $post_categories[0]->cat_ID ) ) {
                    $elements = array_filter( explode( ',', get_category_parents( $post_categories[0]->cat_ID, true, ',' ) ) );
                    for ( $i = 0; $i < count( $elements ); $i++ ) {
                        $breadcrumbs[ 'element_'.$i ] = [ 'name' => '', 'is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'link' => '', 'is_active' => false, 'element' => $elements[$i] ];
                    }
                }
                $breadcrumbs[ 'post' ] = [ 'name' => get_the_title(), 'is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'link' => '', 'is_active' => true, 'element' => '' ];

            } elseif ( get_post_type() === 'page' && !is_archive() ) {

                $parent_pages = [];

                // Set parent pages level by default 1
                if ( empty( $parent_pages_level ) ) { $parent_pages_level = 1; }

                // if page has parent pages
                global $post;
                if ( $post->post_parent ) {
                    $parent_id = $post->post_parent;
                    while ( $parent_id ) {
                        $page = get_post( $parent_id );
                        $parent_pages[] = [ 'name' => get_the_title( $page->ID ), 'is_homepage' => false, 'is_child' => false, 'link' => get_permalink( $page->ID ), 'is_parent' => true, 'is_active' => false, 'element' => '' ];
                        $parent_id = $page->post_parent;
                    }

                    // Get parent pages by nesting level
                    $filtered_parent_pages = [];

                    for ( $i = 0; $i < count( $parent_pages ); $i++ ) {
                        if ( $i === (int)$parent_pages_level || !isset( $parent_pages[$i] ) ) { break; }
                        $filtered_parent_pages[] = $parent_pages[$i];
                    }

                    $parent_pages = array_reverse( $filtered_parent_pages );

                    foreach ( $parent_pages as $parent_page ) {
                        $breadcrumbs[] = $parent_page;
                    }

                }

                $breadcrumbs[ 'page' ] = [ 'name' => get_the_title(), 'is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'is_active' => true, 'link' => '', 'element' => '' ];

                // if page has child pages
                $children = get_children( [ 'post_type' => 'page', 'post_parent' => get_the_ID() ] );
                if ( $children ) {
                    foreach ( $children as $child ) {
                        $breadcrumbs[ 'child_page' ] = [ 'name' => $child->post_title, 'is_homepage' => false, 'is_parent' => false, 'link' => $child->guid, 'is_active' => false, 'element' => '', 'is_child' => true ];
                    }
                }

            } elseif ( is_archive() ) {
                $breadcrumbs[ 'archive' ] = [ 'name' => wp_strip_all_tags( get_the_archive_title() ), 'is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'is_active' => true, 'link' => '', 'element' => '' ];
            } elseif ( is_404() ) {
                $breadcrumbs[ 'Error 404' ] = [ 'name' => 'Error 404', 'is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'is_active' => true, 'link' => '', 'element' => '' ];
            } elseif ( $woocommerce_active ) {
                if ( is_woocommerce() ) {
                    $breadcrumbs['woocommerce'] = ['is_homepage' => false, 'is_parent' => false, 'is_child' => false, 'is_active' => false, 'link' => '', 'element' => ''];
                }
            }
        }

        /** Remove filters from archive title */
        remove_filter( 'get_the_archive_title', [ $this, 'archive_title_filter' ] );
        remove_filter( 'get_the_archive_title_prefix', '__return_false' );

        return $breadcrumbs;
    }

    /**
     * Get data for breadcrumbs ld+json Markup.
     *
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_data_for_schema_breadcrumbs() {

        global $wpdb;

        $result = [];

        $post_id = get_the_ID();

        $data = $wpdb->get_row(
            $wpdb->prepare( "SELECT meta_value FROM {$wpdb->prefix}postmeta WHERE `meta_key` = '_elementor_data'  AND post_id = %d", $post_id )
        );

        if ( !empty( $data ) ) {

            $meta_value = json_decode( $data->meta_value );
            $plugin_id = get_transient( 'plugin_id' );

            foreach ( $meta_value as $item ) {
                foreach ( $item->elements as $element ) {
                    foreach ( $element->elements as $sub_element ) {
                        if ( $sub_element->id === $plugin_id ) {
                            $result['custom_breadcrumbs'] =  isset( $sub_element->settings->custom_breadcrumbs ) ? $sub_element->settings->custom_breadcrumbs : 'no';
                            $result['custom_breadcrumbs_list'] = isset( $sub_element->settings->custom_breadcrumbs_list ) ? $sub_element->settings->custom_breadcrumbs_list : [];
                            $result['exclude_from'] = isset( $sub_element->settings->exclude_from ) ? $sub_element->settings->exclude_from : [];
                            $result['enable_rich_snippet'] = isset( $sub_element->settings->enable_rich_snippet ) && $sub_element->settings->enable_rich_snippet === 'yes' ? 'yes' : 'no';
                            $result['displaying_items'] = isset( $sub_element->settings->displaying_items ) ? $sub_element->settings->displaying_items : [];
                            $result['json_schema_markup'] = isset( $sub_element->settings->json_schema_markup ) ? $sub_element->settings->json_schema_markup : '';
                            $result['custom_homepage_title'] = isset( $sub_element->settings->custom_homepage_title ) ? $sub_element->settings->custom_homepage_title : '';
                            $result['homepage_title'] = isset( $sub_element->settings->homepage_title ) ? $sub_element->settings->homepage_title : '';
                        }
                    }
                }
            }
        }

        return $result;
    }


    /**
     * Creates breadcrumbs item for schema.
     *
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_schema_breadcrumb_item( $value, $name, $breadcrumbs, $position ) {

        if ( $value['link'] === '' && $value['element'] === '' ) {
            $breadcrumbs .= '{
                            "@type": "ListItem",
                            "position": '.( $position + 1 ).',
                            "name": "'.$name.'"
                        },';
        } elseif ( $value['element'] !== '' ) {
            $element = explode('href="', $value['element'] );
            $link = explode( '">', $element[1] );
            $breadcrumbs .= '{
                            "@type": "ListItem",
                            "position": '.( $position + 1 ).',
                            "name": "'.strip_tags( $value['element'] ).'",
                            "item": "'.$link[0].'"
                        },';
        } else {
            $breadcrumbs .= '{
                            "@type": "ListItem",
                            "position": '.( $position + 1 ).',
                            "name": "'.$name.'",
                            "item": "'.$value['link'].'"
                        },';
        }

        return $breadcrumbs;
    }

    /**
     * Checks if woocommerce is active
     *
     * @since 1.0.0
     * @access public
     *
     * @return bool
     **/
    public function is_active_woocommerce() {

        return in_array('woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) );

    }

    /**
     * Output ld+json Markup.
     *
     * @since 1.0.0
     * @access public
     * @return void
     */
    public function structured_data_breadcrumbs() {
        $data = $this->get_data_for_schema_breadcrumbs();

        /** Exit on the Category and Tag pages */
        if ( is_category() || is_tag() ) { return; }

        if ( count( $data ) > 0 ) {

            $breadcrumbs = '';

            if ( !empty( $data['exclude_from'] ) ) {
                foreach ( $data['exclude_from'] as $type ) {
                    if ( $type === 'archive' && is_archive() ) {
                        return;
                    } else {
                        if ( get_post_type() === $type ) {
                            return;
                        }
                    }
                }
            }

            if ( $data['enable_rich_snippet'] === 'no' ) {
                return;
            }

            if ( $data['custom_breadcrumbs'] === 'yes' ) {
                $custom_breadcrumbs = $data['custom_breadcrumbs_list'];

                for ( $i = 0; $i < count( $custom_breadcrumbs ); $i++ ) {

                    if ( !isset( $custom_breadcrumbs[$i]->custom_breadcrumb_link->url ) ) {
                        $breadcrumbs .= '{
                            "@type": "ListItem",
                            "position": '.( $i + 1 ).',
                            "name": "'.$custom_breadcrumbs[$i]->custom_breadcrumb_name.'"
                        },';
                    } else {
                        $breadcrumbs .= '{
                            "@type": "ListItem",
                            "position": '.( $i + 1 ).',
                            "name": "'.$custom_breadcrumbs[$i]->custom_breadcrumb_name.'",
                            "item": "'.$custom_breadcrumbs[$i]->custom_breadcrumb_link->url.'"
                        },';
                    }

                }

            } else {
                $breadcrumbs_arr = $this->get_data_breadcrumbs( $data['parent_pages_level'] );
                $position = 0;

                foreach ( $breadcrumbs_arr as $key => $value ) {
                    if ( !empty( $data['displaying_items'] ) && in_array( 'show_homepage', $data['displaying_items'] ) && $value['is_homepage'] ) {
                        $breadcrumbs = $this->create_schema_breadcrumb_item( $value, $data['custom_homepage_title'] === 'yes' ? $data['homepage_title'] : $value['name'], $breadcrumbs, $position );
                        $position++;
                    }

                    if ( !empty( $data['displaying_items'] ) && in_array( 'show_parent_page', $data['displaying_items'] ) && $value['is_parent'] ) {
                        $breadcrumbs = $this->create_schema_breadcrumb_item( $value, $value['name'], $breadcrumbs, $position );
                        $position++;
                    }

                    if ( !empty( $data['displaying_items'] ) && in_array( 'show_child_pages', $data['displaying_items'] ) && $value['is_child'] ) {
                        $breadcrumbs = $this->create_schema_breadcrumb_item( $value, $value['name'], $breadcrumbs, $position );
                        $position++;
                    }

                    if ( !$value['is_homepage'] && !$value['is_parent'] && !$value['is_child'] ) {
                        $breadcrumbs = $this->create_schema_breadcrumb_item( $value, $value['name'], $breadcrumbs, $position );
                        $position++;
                    }

                }

            }

            $json_ld = $data[ 'json_schema_markup' ];

            $json_ld = str_replace( [ '[breadcrumbs]' ], [ rtrim( $breadcrumbs, ',' ) ], $json_ld );

            printf( '<script type="application/ld+json">%s</script>', $json_ld );

        }

    }

	/**
	 * Render snackbar about duplicated elementor widgets as single plugin by merkulove
	 */
	public function render_snackbar_duplicate() {

		if ( empty( self::$duplicates ) ) { return; }

		$duplicates = preg_filter('/$/', ' for Elementor', self::$duplicates );
		UI::get_instance()->render_snackbar(
			esc_html__( 'Selection already uses the functionality of these plugins: ', 'selection' ) . implode( ', ', $duplicates ),
			'warning',
			-1,
			true,
			[
                [
                    'caption' => esc_html__( 'Plugins', 'selection' ),
                    'link' => admin_url( 'plugins.php' )
                ]
            ]
		);

    }

	/**
	 * Add custom category.
	 *
	 * @param $elements_manager
	 * @since 1.0.0
	 * @return void
	 */
	public function add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'selection-category',
			[
				'title' => esc_html__( 'Selection', 'selection' ),
				'icon' => 'fa fa-plug',
			]
		);

	}

	/**
	 * Scripts and Styles for the Template edit page
	 *
	 * @since   1.0.0
	 * @return void
	 **/
	public function enqueue_edit() {

		$screen = get_current_screen();

		// Run only on Elementor Templates
		if ( null === $screen ) { return; }
		if ( 'elementor_library' !== $screen->id ) { return; }

		// Add class .mdc-disable to body. So we can use UI without overrides WP CSS, only for this page
		add_action( 'admin_body_class', [$this, 'add_admin_class'] );

		// Enqueue styles
		wp_enqueue_style( 'merkulov-ui', Plugin::get_url() . 'src/Merkulove/Unity/assets/css/merkulov-ui' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );
		wp_enqueue_style( 'mdp-selection-edit', Plugin::get_url() . 'css/admin-edit' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );

		// Enqueue scripts
		wp_enqueue_script( 'merkulov-ui', Plugin::get_url() . 'src/Merkulove/Unity/assets/js/merkulov-ui' . Plugin::get_suffix() . '.js', [], Plugin::get_version(), true );
		wp_enqueue_script( 'mdp-selection-edit', Plugin::get_url() . 'js/assignments' . Plugin::get_suffix() . '.js', ['jquery'], Plugin::get_version(), true );

		/** Add code editor for Custom PHP. */
		wp_enqueue_code_editor( array( 'type' => 'application/x-httpd-php' ) );

	}

	/**
	 * Render Activation message.
	 */
	public function not_activated_notice() {

        /** Check activation status */
        if ( TabActivation::get_instance()->is_activated() ) { return; }

		/** Get current screen. */
		$screen = get_current_screen();
		if ( null === $screen ) { return; }

		if ( in_array( $screen->base, Plugin::get_menu_bases() ) ) { //Show only on Settings Page

			/** Render activation message. */
			UI::get_instance()->render_snackbar(
				esc_html__( 'Activate your license to gain access to additional features of the Selection', 'selection' ),
				'warning', // Type
				-1, // Timeout
				true, // Is Closable
				[ [ 'caption' => 'Activate', 'link' => get_admin_url('admin', 'admin.php?page=mdp_selection_settings&tab=activation' ) ] ] // Buttons
			);

		} else {

            /** Render everywhere in admin area */
			?>
            <div class="settings-error notice notice-warning">
                <p>
                    <strong><?php esc_html_e( 'Selection for Elementor', 'selection' ); ?></strong>:
                    <a href="<?php echo get_admin_url('admin', 'admin.php?page=mdp_selection_settings&tab=activation' ) ?>">
						<?php esc_html_e( 'Activate your license', 'selection' ); ?>
                    </a>
	                <?php esc_html_e( ' to gain access to additional features of the Selection', 'selection' ); ?>
                </p>
            </div>
			<?php

        }

	}

	/**
	 * Add class to body in admin area.
	 *
	 * @param string $classes - Space-separated list of CSS classes.
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function add_admin_class( $classes ) {

		return $classes . ' mdc-disable ';

	}

    /**
     * Register WooCommerce frontend
     *
     * @since 1.0.0
     * @access public
     */
    public function register_wc_frontend() {
        wc()->frontend_includes();
    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
