<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building the most awesome websites.
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use Merkulove\Selection\Unity\Plugin;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

final class Elementor {

	/**
	 * The one true Elementor.
	 *
	 * @var Elementor
	 **/
	private static $instance;

	/**
	 * @return void
	 */
    public function setup() {

        /** Define hooks that runs on both the front-end and the dashboard. */
	    $this->register_elementor_widgets();

    }

    /**
     * Registers a Elementor Widget.
     *
     * @return void
     * @access public
     **/
    public function register_elementor_widgets() {

        /** Check for basic requirements. */
        $this->initialization();

        /** Elementor widget Editor CSS. */
        add_action( 'elementor/editor/before_enqueue_styles', [$this, 'editor_styles'] );

    }

	/**
	 * Add our css to admin editor.
	 *
	 * @access public
	 **/
	public function editor_styles() {

		wp_enqueue_style( 'mdp-selection-admin', Plugin::get_url() . 'css/elementor-admin' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );

		$nth = 0;

		$inline_css = '';
		foreach ( TabWidgets::$widget_categories as $category => $addons ) {

			$wrapper_selector = '#elementor-panel-category-selection-category .elementor-panel-category-items';
			$nth_selector = $nth === 0 ? ':first-child' : ':nth-child(' . $nth . ')';

			$inline_css .= $wrapper_selector . ' > div' . $nth_selector . ':before {
			content: "'. esc_html( $category ) .'";
		}';

			$nth === 0 ?
				$nth = 1 + count( $addons ) :
				$nth += count( $addons );

		}

//		wp_add_inline_style( 'mdp-selection-admin', $inline_css ); TODO:Make this more pretty and release

	}

	/**
	 * The init process check for basic requirements and then run the plugin logic.
	 *
	 * @access public
	 **/
	public function initialization() {

		/** Check if Elementor installed and activated. */
		if ( ! did_action( 'elementor/loaded' ) ) { return; }

		/** Register custom widgets. */
		add_action( 'elementor/widgets/widgets_registered', [$this, 'register_widgets'] );

	}

	/**
	 * Register new Elementor widgets.
	 *
	 * @access public
     * @since 1.0.0
     *
     * @return void
     **/
	public function register_widgets() {

		/** @var array $options Shorthand for options */
		$options = get_option( 'mdp_selection_widgets_settings' );

		/** Get list of the Selection widgets from settings */
		$widgets = is_array( $options ) ? $options : array();

		$all_widgets = $this->get_addons();

		$this->require_addons( $all_widgets );

		foreach ( $all_widgets as $name ) {

			$class_name = 'Merkulove\Selection\\' . str_replace( '-', '_', $name ) . '_elementor';

			/** @var string $widget_key - Prepare widget key */
			$widget_key = $this->name_filter( $name );
			$is_widget = isset( $options[ $widget_key ] ) && $options[ $widget_key ] === 'on';

			/** Register Elementor widgets */
			if ( isset( $options[ $widget_key ] ) ) {

				/** Register widgets with 'ON' status */
				if ( $is_widget ) {

					/** Instantiate 'ON' widget and register it in Elementor. */
					\Elementor\Plugin::instance()->widgets_manager->register( new $class_name() );

				}

			} else {

				/** Instantiate 'NEW' widget and register it in Elementor. */
				\Elementor\Plugin::instance()->widgets_manager->register( new $class_name() );

				$widgets[ $widget_key ] = "on";

			}



		}

		/** Update option if widgets list changed */
		if ( $options !== $widgets ) {

			update_option( 'mdp_selection_widgets_settings', $widgets );

		}

	}

	/**
	 * Get all addons names in one array
	 * @return array
	 */
	private function get_addons() {

		$addons = array();

		foreach ( TabWidgets::$widget_categories as $category => $widgets_in_category ) {

			switch ( $category ) {

				case 'woocommerce':

					// Exclude wooCommerce widgets category
					if ( ! Caster::get_instance()->is_active_woocommerce() ) {
						$addons = array_merge( $addons, [ 'couponer' ] );
					} else {
						$addons = array_merge( $addons, $widgets_in_category );
					}
					break;

				default:

					$addons = array_merge( $addons, $widgets_in_category );

			}

		}

		return $addons;

	}

	/**
	 * Require once class for each addons in list
	 * @param $addons
	 *
	 * @return void
	 */
	private function require_addons( $addons ) {

		foreach ( $addons as $name ) {

			$file_name = str_replace( '-', '.', $name );
			require_once( __DIR__ . '/Elementor/widgets/Selection/' . $file_name . '.elementor.php' );

		}

	}

	/**
	 * Renames or replaces in widget class names
	 * @param $class_name
	 *
	 * @return array|string|string[]
	 */
	private function name_filter( $class_name ) {

		/** @var string $widget_key - Prepare widget key */
		$widget_key = str_replace( '.', '-', $class_name );
		$widget_key = str_replace( 'graphist-', '', $widget_key );
		$widget_key = str_replace( '-elementor', '', $widget_key );

		return $widget_key;

	}

	/**
	 * Main Elementor Instance.
	 *
	 * Insures that only one instance of Elementor exists in memory at any one time.
	 *
	 * @static
     *
	 * @return Elementor
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
