<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Galerie for Elementor
 * Customizable gallery for Elementor editor
 * Exclusively on https://1.envato.market/galerie-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Repeater;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Galerie - Custom Elementor Widget.
 **/
class galerie_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-selection-elementor-admin',
               UnityPlugin::get_url() . 'css/elementor-admin' .
                   UnityPlugin::get_suffix() . '.css',
                   [], UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-galerie',
            UnityPlugin::get_url() . 'css/galerie-elementor' . UnityPlugin::get_suffix() . '.css',
                [],
                UnityPlugin::get_version()
        );
	    wp_register_script(
            'mdp-galerie',
               UnityPlugin::get_url() . 'js/galerie-elementor' . UnityPlugin::get_suffix() . '.js',
                   [ 'jquery', 'imagesloaded' ],
                   UnityPlugin::get_version(),
          true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-galerie-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Tabbed Gallery', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-galerie-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Tabbed Gallery' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-galerie', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-galerie' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Gallery Content Section. */
        $this->section_content_gallery();

        /** Content -> Overlay Content Section. */
        $this->section_content_overlay();

        /** Content -> Filter Content Section. */
        $this->section_content_filter();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Filter. */
        $this->section_style_filter();

        /** Style -> Section Style Image. */
        $this->section_style_image();

        /** Style -> Section Style Filter Bar. */
        $this->section_style_filter_bar();

        /** Style -> Section Style Content. */
        $this->section_style_content();

        /** Style -> Section Style Overlay. */
        $this->section_style_overlay();

        /** Style -> Section Style Overlay Title. */
        $this->section_style_overlay_title();

        /** Style -> Section Style Overlay Description. */
        $this->section_style_overlay_description();

    }


    /**
     * Method for generating animation controls.
     *
     * @param $heading
     * @param $slug
     * @param $selectors
     * @param $options
     * @param string $animation_condition
     * @param string $additional_condition
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function create_animation_controls( $heading, $slug, $selectors, $options, $animation_condition = '', $additional_condition = '' ) {

        $anim_options = [
            'label'     => esc_html__( $heading, 'selection' ),
            'type'      => Controls_Manager::SELECT,
            'options'   => $options,
            'default'   => 'none',
        ];

        if ( !empty( $selectors['animation'] ) ) { $anim_options['selectors'] = $selectors['animation']; }
        if ( !empty( $animation_condition ) ) { $anim_options['condition'] = [$animation_condition => 'none' ]; }

        $condition = !empty( $additional_condition ) ?
            [$slug . '_animation!' => 'none', $additional_condition => 'none'] :
            [$slug . '_animation!' => 'none'];

        $this->add_control(
            $slug . '_animation',
            $anim_options
        );

        $this->add_control(
            $slug . '_animation_easing',
            [
                'label' => esc_html__( 'Easing', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'ease',
                'options' => [
                    'ease' => esc_html__( 'Ease', 'selection' ),
                    'ease-in' => esc_html__( 'Ease-in', 'selection' ),
                    'ease-out' => esc_html__( 'Ease-out', 'selection' ),
                    'ease-in-out' => esc_html__( 'Ease-in-out', 'selection' ),
                    'linear' => esc_html__( 'Linear', 'selection' ),
                ],
                'condition' => $condition,
                'selectors' => $selectors['easing']
            ]
        );

        $this->add_control(
             $slug . '_animation_duration',
            [
                'label' => esc_html__( 'Duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 1,
                ],
                'condition' => $condition,
                'selectors' => $selectors['duration']
            ]
        );

        $this->add_control(
            $slug . '_animation_delay',
            [
                'label' => esc_html__( 'Delay', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'condition' => $condition,
                'selectors' => $selectors['delay']
            ]
        );
    }

    /**
     * Add widget controls: Content -> Gallery Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_gallery() {

        $this->start_controls_section( 'section_content_gallery', [
            'label' => esc_html__( 'Gallery', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );


        $this->add_control(
            'gallery_type',
            [
                'label'     => esc_html__( 'Gallery type', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'custom-gallery' => esc_html__( 'Custom', 'selection' ),
                    'post-gallery'   => esc_html__( 'Gallery from post images', 'selection' ),
                ],
                'default'   => 'custom-gallery',
            ]
        );

        $posts = get_posts(
            [
                'post_type' => 'post',
                'post_status' => 'publish',
                'ignore_sticky_posts' => 1,
                'numberposts' => -1
            ]
        );

        $options = wp_list_pluck( $posts, 'post_title', 'ID' );

        $this->add_control(
            'posts',
            [
                'label' => esc_html__( 'Posts', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $options,
                'condition' => [ 'gallery_type' => 'post-gallery' ]
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'gallery_title',
            [
                'label' => esc_html__( 'Gallery title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Gallery', 'selection' ),
                'placeholder' => esc_html__( 'Type your title here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'gallery',
            [
                'label' => esc_html__( 'Add Images', 'selection' ),
                'type' => Controls_Manager::GALLERY,
                'default' => [],
            ]
        );

        $this->add_control(
            'galleries',
            [
                'label' => esc_html__( 'Galleries', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'gallery_title' => esc_html__( 'Gallery', 'selection' ),
                    ],
                ],
                'title_field' => '{{{ gallery_title }}}',
                'condition' => [
                    'gallery_type' => 'custom-gallery'
                ]
            ]
        );

        $this->add_control(
            'gallery_layout',
            [
                'label'     => esc_html__( 'Layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'grid' => esc_html__( 'Grid', 'selection' ),
                    'masonry'   => esc_html__( 'Masonry', 'selection' ),
                ],
                'default'   => 'grid',
            ]
        );

        $this->add_responsive_control(
            'gallery_columns',
            [
                'label' => esc_html__( 'Columns', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 20,
                'step' => 1,
                'default' => 1,
            ]
        );

        $this->add_control(
            'link_type',
            [
                'label'     => esc_html__( 'Link', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'none' => esc_html__( 'None', 'selection' ),
                    'media'   => esc_html__( 'Media', 'selection' ),
                    'custom'   => esc_html__( 'Custom', 'selection' ),
                    'post-link'   => esc_html__( 'Post link', 'selection' )
                ],
                'default'   => 'media',
            ]
        );


        $this->add_control(
            'post_link_new_tab',
            [
                'label' => esc_html__( 'Open link in new tab', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'link_type' => 'post-link'
                ]
            ]
        );

        $this->add_control(
            'custom_link',
            [
                'label' => esc_html__( 'Custom link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
                'condition' => [
                    'link_type' => 'custom',
                ]
            ]
        );

        $this->add_control(
            'gallery_image_overlay',
            [
                'label' => esc_html__( 'Overlay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

       $this->add_control(
            'gallery_image_lazy_load',
            [
                'label' => esc_html__( 'Lazy load', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

       $animation_options = [
           'none' => esc_html__( 'None', 'selection' ),
           'grow-hover'   => esc_html__( 'Grow', 'selection' ),
           'shrink-hover'   => esc_html__( 'Shrink', 'selection' ),
           'slide-up-hover'   => esc_html__( 'Slide up', 'selection' ),
           'slide-down-hover'   => esc_html__( 'Slide down', 'selection' ),
           'swing-hover'   => esc_html__( 'Swing', 'selection' )
       ];

       $selectors = [
           'easing' => [
               '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover' => 'animation-timing-function: {{VALUE}} !important'
           ],
           'duration' => [
               '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover' => 'animation-duration: {{SIZE}}{{UNIT}} !important'
           ],
           'delay' => [
               '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover' => 'animation-delay: {{SIZE}}{{UNIT}} !important'
           ]
       ];

       $this->create_animation_controls( 'Image animation', 'gallery_image', $selectors, $animation_options );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Overlay Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_overlay()
    {
        $this->start_controls_section( 'section_content_overlay', [
            'label' => esc_html__( 'Overlay', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'gallery_image_overlay' => 'yes',
            ]
        ] );

        $this->add_control(
            'overlay_title_type',
            [
                'label'     => esc_html__( 'Title', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'none' => esc_html__( 'None', 'selection' ),
                    'title'   => esc_html__( 'Title', 'selection' ),
                    'caption'   => esc_html__( 'Caption', 'selection' ),
                    'alt'   => esc_html__( 'Alt', 'selection' ),
                    'description'   => esc_html__( 'Description', 'selection' ),
                    'custom'   => esc_html__( 'Custom', 'selection' ),
                ],
                'default'   => 'none',
            ]
        );

        $this->add_control(
            'overlay_custom_title',
            [
                'label' => esc_html__( 'Custom title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your title here', 'selection' ),
                'condition' => [
                    'overlay_title_type' => 'custom',
                ]
            ]
        );

        $this->add_control(
            'overlay_description_type',
            [
                'label'     => esc_html__( 'Description', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'none' => esc_html__( 'None', 'selection' ),
                    'title'   => esc_html__( 'Title', 'selection' ),
                    'caption'   => esc_html__( 'Caption', 'selection' ),
                    'alt'   => esc_html__( 'Alt', 'selection' ),
                    'description'   => esc_html__( 'Description', 'selection' ),
                    'custom'   => esc_html__( 'Custom', 'selection' ),
                ],
                'default'   => 'none',
            ]
        );

        $this->add_control(
            'overlay_custom_description',
            [
                'label' => esc_html__( 'Custom description', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
                'placeholder' => esc_html__( 'Type your description here', 'selection' ),
                'condition' => [
                    'overlay_description_type' => 'custom',
                ]
            ]
        );

        $this->add_control(
            'overlay_content_alignment',
            [
                'label' => esc_html__( 'Text alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-image-overlay-content' => 'align-items: {{VALUE}}'
                ],
                'default' => 'flex-start',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'overlay_content_vertical_position',
            [
                'label' => esc_html__( 'Vertical position', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Top', 'selection' ),
                        'icon' => 'eicon-v-align-top',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-v-align-middle',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Bottom', 'selection' ),
                        'icon' => 'eicon-v-align-bottom',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-image-overlay-content' => 'justify-content: {{VALUE}}'
                ],
                'default' => 'flex-start',
                'toggle' => true,
            ]
        );

        //animation controls for overlay
        $options = [
            'none' => esc_html__( 'None', 'selection' ),
            'grow'   => esc_html__( 'Grow', 'selection' ),
            'shrink'   => esc_html__( 'Shrink', 'selection' ),
            'slide-in-right'   => esc_html__( 'Slide in right', 'selection' ),
            'slide-in-left'   => esc_html__( 'Slide in left', 'selection' ),
            'slide-in-up'   => esc_html__( 'Slide in up', 'selection' ),
            'slide-in-down'   => esc_html__( 'Slide in down', 'selection' ),
            'fade'   => esc_html__( 'Fade in', 'selection' )
        ];

        $options_overlay = [
            'none' => esc_html__( 'None', 'selection' ),
            'grow'   => esc_html__( 'Grow', 'selection' ),
            'slide-in-right'   => esc_html__( 'Slide in right', 'selection' ),
            'slide-in-left'   => esc_html__( 'Slide in left', 'selection' ),
            'slide-in-up'   => esc_html__( 'Slide in up', 'selection' ),
            'slide-in-down'   => esc_html__( 'Slide in down', 'selection' ),
            'fade'   => esc_html__( 'Fade in', 'selection' )
        ];

        $selectors_overlay_anim = [
            'animation' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay' => 'animation: {{VALUE}} both !important',
            ],
            'easing' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay' => 'animation-timing-function: {{VALUE}} !important',
            ],
            'duration' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay' => 'animation-duration: {{SIZE}}{{UNIT}} !important',
            ],
            'delay' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay' => 'animation-delay: {{SIZE}}{{UNIT}} !important',
            ]
        ];

        $this->create_animation_controls(
            'Overlay animations',
            'overlay',
            $selectors_overlay_anim,
            $options_overlay
        );

        // animation controls for overlay text
        $selectors = [
            'animation' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay-content' => 'animation: {{VALUE}} both !important',
            ],
            'easing' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay-content' => 'animation-timing-function: {{VALUE}} !important',
            ],
            'duration' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay-content' => 'animation-duration: {{SIZE}}{{UNIT}} !important',
            ],
            'delay' => [
                '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover .mdp-galerie-elementor-image-overlay-content' => 'animation-delay: {{SIZE}}{{UNIT}} !important',
            ],
        ];

        $this->create_animation_controls(
            'Overlay text animations',
            'overlay_text',
            $selectors,
            $options
        );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Filter Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_filter()
    {
        $this->start_controls_section( 'section_content_filter', [
            'label' => esc_html__( 'Filter', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
        ] );

        $this->add_control(
            'filter_alignment',
            [
                'label' => esc_html__( 'Filter alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'all_galleries_filter',
            [
                'label' => esc_html__( 'All galleries filter', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'all_filter_name',
            [
                'label' => esc_html__( '"All" filter name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your title here', 'selection' ),
                'default' => esc_html__( 'All', 'selection' ),
                'condition' => [
                    'all_galleries_filter' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'filter_item_pointer',
            [
                'label'     => esc_html__( 'Pointer', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'mdp-galerie-elementor-filter-pointer-underline' => esc_html__('Underline', 'selection'),
                    'mdp-galerie-elementor-filter-pointer-overline' => esc_html__( 'Overline', 'selection' ),
                    'mdp-galerie-elementor-filter-pointer-double-line' => esc_html__( 'Double line', 'selection' ),
                    'mdp-galerie-elementor-filter-pointer-framed' => esc_html__( 'Framed', 'selection' ),
                    'none' => esc_html__( 'None', 'selection' ),
                ],
                'default'   => 'none',
            ]
        );

        // animation controls for pointer
        $selectors = [
            'animation' => [
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-underline:hover::after" => 'animation: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-overline:hover::after" => 'animation: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::after" => 'animation: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::before" => 'animation: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-framed:hover::before" => 'animation: {{VALUE}}',
            ],
            'easing' => [
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-underline:hover::after" => 'animation-timing-function: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-overline:hover::after" => 'animation-timing-function: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::after" => 'animation-timing-function: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::before" => 'animation-timing-function: {{VALUE}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-framed:hover::before" => 'animation-timing-function: {{VALUE}}',
            ],
            'duration' => [
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-underline:hover::after" => 'animation-duration: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-overline:hover::after" => 'animation-duration: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::after" => 'animation-duration: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::before" => 'animation-duration: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-framed:hover::before" => 'animation-duration: {{SIZE}}{{UNIT}}',
            ],
            'delay' => [
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-underline:hover::after" => 'animation-delay: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-overline:hover::after" => 'animation-delay: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::after" => 'animation-delay: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::before" => 'animation-delay: {{SIZE}}{{UNIT}}',
                "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-framed:hover::before" => 'animation-delay: {{SIZE}}{{UNIT}}',
            ]
        ];


        $animation_options = [
            'grow' => esc_html__( 'Grow', 'selection' ),
            'shrink' => esc_html__( 'Shrink', 'selection' ),
            'slide-up' => esc_html__( 'Slide up', 'selection' ),
            'slide-down' => esc_html__( 'Slide down', 'selection' ),
            'fade' => esc_html__( 'Fade', 'selection' ),
            'none' => esc_html__( 'None', 'selection' ),
        ];

        $this->create_animation_controls(
            'Pointer animation',
             'filter_item_pointer',
                  $selectors,
                  $animation_options,
                  'filter_item_pointer!',
                  'filter_item_pointer!'
        );

        // animation controls for text
        $selectors_text_anim = [
            'easing' => [
                '{{WRAPPER}} .mdp-galerie-elementor-filter-item:hover' => 'animation-timing-function: {{VALUE}} !important'
            ],
            'duration' => [
                '{{WRAPPER}} .mdp-galerie-elementor-filter-item:hover' => 'animation-duration: {{SIZE}}{{UNIT}} !important'
            ],
            'delay' => [
                '{{WRAPPER}} .mdp-galerie-elementor-filter-item:hover' => 'animation-delay: {{SIZE}}{{UNIT}} !important'
            ],
        ];

        $options_text_anim = [
            'none' => esc_html__( 'None', 'selection' ),
            'grow-hover'   => esc_html__( 'Grow', 'selection' ),
            'shrink-hover'   => esc_html__( 'Shrink', 'selection' ),
            'slide-up-hover'   => esc_html__( 'Slide up', 'selection' ),
            'slide-down-hover'   => esc_html__( 'Slide down', 'selection' ),
            'swing-hover'   => esc_html__( 'Swing', 'selection' )
        ];

        $this->create_animation_controls(
            'Text animations',
            'filter_text',
            $selectors_text_anim,
            $options_text_anim,
            'filter_item_pointer',
            'filter_item_pointer'
        );

        $this->end_controls_section();
    }

    /**
     * Method for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {


        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_color_hover' => array_key_exists( 'additional_color_hover', $opts ) ?
                $opts['additional_color_hover'] : false,
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'include_css_filters' => array_key_exists( 'include_css_filters', $opts ) ?
                $opts['include_css_filters'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'ACTIVE',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_selectors' => array_key_exists( 'additional_color_hover_selectors', $opts ) ?
                $opts['additional_color_hover_selectors'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'additional_border_radius_selector' => array_key_exists( 'additional_border_radius_selector', $opts ) ?
                $opts['additional_border_radius_selector'] : '',
            'additional_border_radius_hover_selector' => array_key_exists( 'additional_border_radius_hover_selector', $opts ) ?
                $opts['additional_border_radius_hover_selector'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'],
                ]
            );
        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color_hover'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' =>  $style_opts['additional_color_hover_selectors'],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'].':hover',
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'galerie-elementor '),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_active',
                    'label' => esc_html__( 'Border Type', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Filter.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_filter() {

        $this->start_controls_section( 'section_style_filter', [
            'label' => esc_html__( 'Filter', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_filter',
            'mdp-galerie-elementor-filter-item'
        );

        $this->add_control(
            'filter_space_between_items_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_responsive_control(
            'filter_space_between_items',
            [
                'label' => esc_html__( 'Space between items', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-filter-item' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                    '{{WRAPPER}} .mdp-galerie-elementor-filter-item:first-child' => 'margin-left: calc({{SIZE}}{{UNIT}} * 0);',
                    '{{WRAPPER}} .mdp-galerie-elementor-filter-item:last-child' => 'margin-right: calc({{SIZE}}{{UNIT}} * 0);',
                ],
            ]
        );


        $pointer_selectors = [
            "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-underline:hover::after" => 'background-color: {{VALUE}}',
            "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-overline:hover::after" => 'background-color: {{VALUE}}',
            "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::after" => 'background-color: {{VALUE}}',
            "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-double-line:hover::before" => 'background-color: {{VALUE}}',
            "{{WRAPPER}} .mdp-galerie-elementor-filter-pointer-framed:hover::before" => 'border-color: {{VALUE}}',
        ];

        $this->generate_typography_tabs_controls( 'section_style_filter', [
            'html_class' => 'mdp-galerie-elementor-filter-item',
            'additional_color_hover' => true,
            'include_active_tab' => true,
            'active_class' => 'mdp-galerie-elementor-filter-item-active',
            'color_active_class' => 'mdp-galerie-elementor-filter-item-active',
            'additional_color_name' => 'Pointer color',
            'additional_color_hover_selectors' => $pointer_selectors,
            'color_class' => 'mdp-galerie-elementor-filter-item',
            'color_hover_class' => 'mdp-galerie-elementor-filter-item:hover',
            'typography_class' => 'mdp-galerie-elementor-filter-item',
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_image() {

        $this->start_controls_section( 'section_style_image', [
            'label' => esc_html__( 'Image', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_image',
            'mdp-galerie-elementor-gallery-image'
        );

        $this->add_responsive_control(
            'image_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-gallery-image' => 'width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-galerie-elementor-gallery-image img' => 'width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-galerie-elementor-gallery-masonry-column' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'image_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'condition' => [
                    'gallery_layout' => 'grid'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-gallery-image img' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'image_height_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_image', [
            'html_class' => 'mdp-galerie-elementor-gallery-image',
            'additional_border_radius_hover_selector' => '{{WRAPPER}} .mdp-galerie-elementor-gallery-image:hover img',
            'additional_border_radius_selector' => '{{WRAPPER}} .mdp-galerie-elementor-gallery-image img',
            'include_typography' => false,
            'include_color' => false,
            'include_css_filters' => true,
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Content.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_content() {

        $this->start_controls_section( 'section_style_content', [
            'label' => esc_html__( 'Content', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_content',
            'mdp-galerie-elementor-content'
        );

        $this->add_control(
            'content_space_between_items_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_responsive_control(
            'content_space_between_items',
            [
                'label' => esc_html__( 'Space between items', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 20,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-masonry-layout, {{WRAPPER}} .mdp-galerie-elementor-grid-layout' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-galerie-elementor-gallery-masonry-column, {{WRAPPER}} .mdp-galerie-elementor-grid-layout' => 'row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'content_space_between_items_divider_bottom',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_content', [
            'html_class' => 'mdp-galerie-elementor-content',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Filter Bar.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_filter_bar() {

        $this->start_controls_section( 'section_style_filter_bar', [
            'label' => esc_html__( 'Filter bar', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_filter_bar',
            'mdp-galerie-elementor-filter-nav'
        );


        $this->generate_typography_tabs_controls( 'section_style_filter_bar', [
            'html_class' => 'mdp-galerie-elementor-filter-nav',
            'include_typography' => false,
            'include_color' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Overlay.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay() {
        $this->start_controls_section( 'section_style_overlay', [
            'label' => esc_html__( 'Overlay', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'gallery_image_overlay' => 'yes',
            ]
        ] );

        $this->add_control(
            'overlay_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-image-overlay' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'overlay_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    '' => esc_html__( 'Normal', 'selection' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-galerie-elementor-image-overlay' => 'mix-blend-mode: {{VALUE}}',
                ],
                'separator' => 'none',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'overlay_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .mdp-galerie-elementor-image-overlay',
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'border',
                'label' => esc_html__( 'Border', 'selection' ),
                'separator' => 'before',
                'selector' => '{{WRAPPER}} .mdp-galerie-elementor-image-overlay',
            ]
        );

        $this->add_responsive_control(
            'overlay_border_radius',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-galerie-elementor-image-overlay" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Overlay Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay_title() {
        $this->start_controls_section('section_style_overlay_title', [
            'label' => esc_html__('Overlay title', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'gallery_image_overlay' => 'yes',
                'overlay_title_type!' => 'none'
            ]
        ]);

        $this->generate_margin_padding_controls(
            'section_style_overlay_title',
            'mdp-galerie-elementor-image-overlay-title'
        );

        $this->generate_typography_tabs_controls( 'section_style_overlay_title', [
            'html_class' => 'mdp-galerie-elementor-image-overlay-title',
            'typography_class' => 'mdp-galerie-elementor-image-overlay-title',
            'color_class' => 'mdp-galerie-elementor-image-overlay-title',
            'color_hover_class' => 'mdp-galerie-elementor-image-overlay-title:hover',
        ] );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Overlay Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay_description() {
        $this->start_controls_section('section_style_overlay_description', [
            'label' => esc_html__('Overlay description', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'gallery_image_overlay' => 'yes',
                'overlay_description_type!' => 'none'
            ]
        ]);

        $this->generate_margin_padding_controls(
            'section_style_overlay_description',
            'mdp-galerie-elementor-image-overlay-description'
        );

        $this->generate_typography_tabs_controls( 'section_style_overlay_description', [
            'html_class' => 'mdp-galerie-elementor-image-overlay-description',
            'typography_class' => 'mdp-galerie-elementor-image-overlay-description',
            'color_class' => 'mdp-galerie-elementor-image-overlay-description',
            'color_hover_class' => 'mdp-galerie-elementor-image-overlay-description:hover',
        ] );


        $this->end_controls_section();

    }

    /**
     * Get posts categories.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_posts_categories( $settings ) {
        $categories = [];
        $result = [];

        if ( empty( $settings['posts'] ) ) { return; }

        foreach ( $settings['posts'] as $post ) {
            $categories[] = get_the_category( $post );
        }

        $categories = array_merge( ...$categories );

        foreach ( $categories as $category ) {
           $result[$category->cat_ID] = $category->name;
        }

        return $result;
    }

    /**
     * Method for generating gallery filter.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function gallery_filter( $settings ) {

        $filter_items = '';
        $filter_alignment_class = 'mdp-galerie-elementor-filter-alignment-' . esc_attr( $settings['filter_alignment'] );
        $pointer_class = $settings['filter_item_pointer'] !== 'none' ?
            esc_attr( $settings['filter_item_pointer'] ) :
            '';

        $galleries = $settings['gallery_type'] === 'custom-gallery' ?
            $settings['galleries'] :
            $this->get_posts_categories( $settings );

        if ( $settings['all_galleries_filter'] === 'yes' ) {
            $filter_items .= sprintf(
                '<div data-index="%s" class="mdp-galerie-elementor-filter-item %s">%s </div>',
                'all',
                        $pointer_class,
                        esc_html( $settings['all_filter_name'] )
            );
        }

        if ( empty( $galleries ) ) { return; }

        foreach ( $galleries as $index => $gallery ) {
            $filter_items .= sprintf(
                '<div data-index="%s" class="mdp-galerie-elementor-filter-item %s">%s </div>',
                esc_attr( $index ),
                $pointer_class,
                esc_html( $settings['gallery_type'] === 'custom-gallery' ? $gallery['gallery_title'] : $gallery )
            );
        }

        return sprintf(
            '<div class="mdp-galerie-elementor-filter-nav">
                      <div class="mdp-galerie-elementor-filter-nav-scroll %s">%s</div>
                    </div>',
                    $filter_alignment_class,
                    $filter_items
        );

    }


    /**
     * Method for generating gallery content.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function gallery_content( $settings ) {
        $masonry_layout = $settings['gallery_layout'] === 'masonry' ?
            esc_attr( 'mdp-galerie-elementor-masonry-layout' ) :
            'mdp-galerie-elementor-grid-layout';

        if ( $settings['link_type'] === 'custom' && !empty( $settings['custom_link'] ) ) {
            $this->add_link_attributes( 'custom_link', $settings['custom_link'] );
        }

        $galleries = $settings['gallery_type'] === 'custom-gallery' ?
            $this->create_custom_galleries( $settings ) :
            $this->create_post_galleries( $settings );


        return sprintf(
            '<div class="mdp-galerie-elementor-content %s">%s</div>',
                   $masonry_layout,
                   $galleries
        );
    }


    /**
     * Get data for overlay.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_overlay_data ( $settings ) {
        return [
            'enable_overlay' => esc_html( $settings['gallery_image_overlay'] ),
            'overlay_title_type' => esc_html( $settings['overlay_title_type'] ),
            'overlay_desc_type' => esc_html( $settings['overlay_description_type'] ),
            'overlay_custom_title' => esc_html( $settings['overlay_custom_title'] ),
            'overlay_custom_description' => wp_kses_post( $settings['overlay_custom_description'] )
        ];
    }

    /**
     * Method for generating custom galleries.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_custom_galleries( $settings ) {
        $link_type = esc_html( $settings['link_type'] );

        $overlay_data = $this->get_overlay_data( $settings );

        $gallery_images = [];

        foreach ( $settings['galleries'] as $index => $gallery ) {
            $gallery_images[] = $this->create_custom_gallery_images(
                $gallery['gallery'],
                $link_type,
                $index
            );

        }

        $gallery_images = array_merge( ...$gallery_images );

        $tmp = [];
        $result = [];

        foreach ( $gallery_images as $gallery_image ) {
            if ( isset( $tmp[$gallery_image['id']] ) ) {
                $tmp[$gallery_image['id']][] = $gallery_image['filter'];
            } else {
                $tmp[$gallery_image['id']] = [ $gallery_image['filter'] ];
            }

            foreach ( $tmp as $key => $value ) {
                if ( $gallery_image['id'] === $key ) {
                    $gallery_image['filter'] = implode( ' ', $tmp[$gallery_image['id']] );
                }
            }

            $result[$gallery_image['id']] = $gallery_image;

        }

        $gallery_images = $result;

        return $this->create_image_html(
            $gallery_images,
            $overlay_data,
            esc_attr( $settings['all_galleries_filter'] ),
            esc_html( $settings['gallery_type'] ),
            esc_html( $settings['gallery_image_lazy_load'] ),
            esc_html( $settings['post_link_new_tab'] )
        );

    }

    /**
     * Method for generating galleries from posts feature images.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_post_galleries( $settings ) {

        $overlay_data = $this->get_overlay_data( $settings );

        $galley_images = $this->create_post_gallery_images( $settings );

        return $this->create_image_html(
            $galley_images,
            $overlay_data,
            esc_attr( $settings['all_galleries_filter'] ),
            esc_html( $settings['gallery_type'] ),
            esc_html( $settings['gallery_image_lazy_load'] ),
            esc_html( $settings['post_link_new_tab'] )
        );
    }

    /**
     * Method for generating gallery overlay title.
     *
     * @param $image_data
     * @param $title_type
     * @param $custom_title
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_overlay_title( $image_data, $title_type, $custom_title ) {
        $title_content = '';

        switch ( $title_type ) {
            case 'title':
                $title_content = $image_data['title'];
            break;
            case 'alt':
                $title_content = $image_data['alt'];
            break;
            case 'caption':
                $title_content = $image_data['caption'];
            break;
            case 'custom':
                $title_content = $custom_title;
            break;
        }

        return sprintf(
            '<div class="mdp-galerie-elementor-image-overlay-title">%s</div>',
                   $title_content
        );

    }

    /**
     * Method for generating gallery overlay description.
     *
     * @param $image_data
     * @param $description_type
     * @param $custom_description
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_overlay_description( $image_data, $description_type, $custom_description ) {
        $description_content = '';

        switch ( $description_type ) {
            case 'title':
                $description_content = $image_data['title'];
            break;
            case 'alt':
                $description_content = $image_data['alt'];
            break;
            case 'caption':
                $description_content = $image_data['caption'];
            break;
            case 'description':
                $description_content = $image_data['description'];
            break;
            case 'custom':
                $description_content = $custom_description;
            break;
        }

        return sprintf(
            '<div class="mdp-galerie-elementor-image-overlay-description">%s</div>',
                   $description_content
        );

    }

    /**
     * Method for generating gallery overlay.
     *
     * @param $image_data
     * @param $overlay_data
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_overlay( $image_data, $overlay_data ) {
        $overlay_title = $this->create_overlay_title(
            $image_data,
            $overlay_data['overlay_title_type'],
            $overlay_data['overlay_custom_title']
        );
        $overlay_description = $this->create_overlay_description(
            $image_data,
            $overlay_data['overlay_desc_type'],
            $overlay_data['overlay_custom_description']
        );

        return sprintf(
            '<div class="mdp-galerie-elementor-image-overlay">
                        <div class="mdp-galerie-elementor-image-overlay-content">%s %s</div>
                    </div>',
            $overlay_title,
            $overlay_description
        );
    }

    /**
     * Method for generating custom galleries images.
     *
     * @param $images
     * @param $link_type
     * @param $index
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function create_custom_gallery_images( $images, $link_type, $index ) {
        $gallery = [];

        foreach ( $images as $image ) {
            $attached_image = get_post( $image['id'] );
            $image_data = [
                'alt' => get_post_meta( $attached_image->ID, '_wp_attachment_image_alt', true ),
                'title' => $attached_image->post_title,
                'caption' => $attached_image->post_excerpt,
                'description' => $attached_image->post_content
            ];

            $gallery[] = [
                'id' => $image['id'],
                'url' => $image['url'],
                'image_data' => $image_data,
                'filter' => $index,
                'link_type' => $link_type,
            ];

        }

        return $gallery;

    }

    /**
     * Method for generating images for post gallery.
     *
     * @param $settings
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function create_post_gallery_images( $settings ) {
        $link_type = esc_html( $settings['link_type'] );

        $gallery_images = [];


        $posts = $settings['posts'];

        foreach ( $posts as $post ) {

            if ( !get_post_thumbnail_id( esc_sql( $post ) ) ) { continue; }

            $post_link = $settings['gallery_type'] === 'post-gallery' ? get_post_permalink( $post ) : '';
            $post_categories = array_map( function ( $item ) { return $item->cat_ID; }, get_the_category( $post ) );
            $attached_image = get_post( get_post_thumbnail_id( $post ) );

            $image_data = [
                'alt' => get_post_meta( $attached_image->ID, '_wp_attachment_image_alt', true ),
                'title' => $attached_image->post_title,
                'caption' => $attached_image->post_excerpt,
                'description' => $attached_image->post_content
            ];

            $gallery_images[] = [
                'id' => get_post_thumbnail_id( esc_sql( $post ) ),
                'url' => wp_get_attachment_image_src( get_post_thumbnail_id( $post ), 'full' ),
                'image_data' => $image_data,
                'filter' => implode( ' ', $post_categories ),
                'link_type' => $link_type,
                'post_link' => $post_link
            ];
        }

        return $gallery_images;

    }

    /**
     * Method for generating custom image html.
     *
     * @param $gallery_images
     * @param $overlay_data
     * @param $all_filter
     * @param $gallery_type
     * @param $lazy_load
     * @param $post_link_new_tab
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_image_html( $gallery_images, $overlay_data, $all_filter, $gallery_type, $lazy_load, $post_link_new_tab ) {
        $gallery = [];
        $image_class = 'mdp-galerie-elementor-gallery-image';

        $lazy_load_class =  $lazy_load === 'yes' ? 'mdp-galerie-elementor-gallery-image-lazy' : '';

        foreach ( $gallery_images as $gallery_image ) {

            $overlay = $overlay_data['enable_overlay'] === 'yes' ?
                $this->create_overlay( $gallery_image['image_data'], $overlay_data ) :
                '';

            if ( $all_filter === 'yes' ) {
                $gallery_image['filter'] .= ' all';
            }

            $image_url = $gallery_type === 'custom-gallery' ? $gallery_image['url'] : $gallery_image['url'][0];

            $src = $lazy_load === 'yes' ? 'data-src=' . esc_url( $image_url ) : 'src=' . esc_url( $image_url );

            if ( $gallery_image['link_type'] === 'none' ) {
                $gallery[] = sprintf(
                    '<div data-filter="%s" class="%s">%s<img class="%s" %s></div>',
                    $gallery_image['filter'],
                    $image_class,
                    $overlay,
                    $lazy_load_class,
                    $src
                );
            } elseif ( $gallery_image['link_type'] === 'media' ) {
                $gallery[] = sprintf(
                    '<a data-filter="%s" class="%s" href="%s" data-elementor-open-lightbox="yes">
                              %s
                              <img class="%s" %s>
                            </a>',
                    esc_attr( $gallery_image['filter'] ),
                    esc_attr( $image_class ),
                    esc_url( $image_url ),
                    $overlay,
                    esc_attr( $lazy_load_class ),
                    $src
                );
            } elseif ( $gallery_image['link_type'] === 'custom' ) {
                $gallery[] = sprintf(
                    '<a data-filter="%s" class="%s" %s>
                              %s
                              <img class="%s" %s>
                            </a>',
                    $gallery_image['filter'],
                    esc_attr( $image_class ),
                    $this->get_render_attribute_string( 'custom_link' ),
                    $overlay,
                    esc_attr( $lazy_load_class ),
                    $src
                );
            } elseif ( $gallery_image['link_type'] === 'post-link' ) {
                $post_link = $gallery_type === 'post-gallery' ? $gallery_image['post_link'] : '';
                $new_tab_link = $post_link_new_tab === 'yes' ? 'target="_blank"' : '';
                $gallery[] = sprintf(
                    '<a data-filter="%s" class="%s" href="%s" %s>
                              %s
                              <img class="%s" %s>
                            </a>',
                    $gallery_image['filter'],
                    esc_attr( $image_class ),
                    esc_url( $post_link ),
                    $new_tab_link,
                    $overlay,
                    esc_attr( $lazy_load_class ),
                    $src
                );
            }
        }

        return implode( ' ', $gallery );

    }


    /**
     * Method for generating columns data attribute.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function get_columns_data_attrs( $settings ) {
        $breakpoints = Plugin::$instance->breakpoints->get_active_breakpoints();
        $attrs = [];

        $attrs[] = 'data-columns="' . esc_attr( $settings['gallery_columns'] ) .'"';

        foreach ( $breakpoints as $key => $value ) {
            if ( isset( $settings['gallery_columns_' . $key] ) ) {
                $attrs[] = 'data-columns-' . $key . '=' . '"' . esc_attr( $settings['gallery_columns_' . $key] ) . '"';
            }
        }

        return implode( ' ', $attrs );
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();

        $this->get_columns_data_attrs( $settings );

        $gallery_filter = $this->gallery_filter( $settings );
        $all_filter = esc_attr( $settings['all_galleries_filter'] );
        $gallery_content = $this->gallery_content( $settings );
        $layout = esc_attr( $settings['gallery_layout'] );
        $columns_attrs = $this->get_columns_data_attrs( $settings );
        $lazy_load = esc_attr( $settings['gallery_image_lazy_load'] );
        $image_animation = esc_attr( $settings['gallery_image_animation'] );
        $filter_text_animation = esc_attr( $settings['filter_text_animation'] );

        echo sprintf(
            ' <!-- Start Galerie for Elementor WordPress Plugin -->
                        <div class="mdp-galerie-elementor-box" 
                        data-layout="%s"
                        data-image-animation="%s"
                        data-filter-text-animation="%s"
                        %s 
                        data-lazy-load="%s"
                        data-all-filter="%s"
                        >
                            <div class="mdp-galerie-elementor-gallery-tabs">%s %s</div>
                        </div>
                        <!-- End Galerie for Elementor WordPress Plugin -->',
            $layout,
            $image_animation,
            $filter_text_animation,
            $columns_attrs,
            $lazy_load,
            $all_filter,
            $gallery_filter,
            $gallery_content
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/galerie';

    }

}
