<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Mapper for Elementor
 * Custom maps for Elementor editor
 * Exclusively on https://1.envato.market/mapper-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Elementor\Repeater;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;
use Merkulove\Selection\Unity\Settings;

/** @noinspection PhpUnused */
/**
 * Mapper - Custom Elementor Widget.
 **/
class mapper_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {
        $options = Settings::get_instance()->options;

        parent::__construct( $data, $args );

        wp_register_style(
     'mdp-selection-elementor-admin',
        UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
         'mdp-mapper-elementor',
            UnityPlugin::get_url() .
                'css/mapper-elementor' . UnityPlugin::get_suffix() . '.css',
                [], UnityPlugin::get_version()
        );
        wp_register_script( 'google-gstatic-geochart', 'https://www.gstatic.com/charts/loader.js' );
        wp_register_script(
          'mdp-mapper-elementor',
            UnityPlugin::get_url() . 'js/mapper-elementor' . UnityPlugin::get_suffix() . '.js',
                [ 'jquery' ],
                UnityPlugin::get_version(),
       true
        );

        wp_localize_script( 'mdp-mapper-elementor', 'mdpMapperObjectWP', [
            'maps_key' => $options['google_maps_api_key']
        ] );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-mapper-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Interactive World Map', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-mapper-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Interactive World Map' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-mapper-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'google-gstatic-geochart', 'mdp-mapper-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Map Settings Content Section. */
        $this->section_content_map_settings();

        /** Content -> Markers Content Section. */
        $this->section_content_markers();

        /** Content -> Regions Content Section. */
        $this->section_content_regions();

        /** Content -> Text Content Section. */
        $this->section_content_text();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Tooltip. */
        $this->section_style_map();

        /** Style -> Section Style Tooltip. */
        $this->section_style_tooltip();

        /** Style -> Section Style Markers Title. */
        $this->section_style_tooltip_title();

        /** Style -> Section Style Marker Description. */
        $this->section_style_tooltip_description();

        /** Style -> Section Style Value. */
        $this->section_style_data_vis_value();

        /** Style -> Section Style Additional Value. */
        $this->section_style_data_vis_additional_value();

        /** Style -> Section Style Overlay. */
        $this->section_style_overlay();

    }

    /**
     * Add widget controls: Content -> Map Settings Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_map_settings() {

        $this->start_controls_section( 'section_content_map_settings', [
            'label' => esc_html__( 'Map settings', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'region_to_display',
            [
                'label' => esc_html__( 'Region to display', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type region here', 'selection' ),
                'description' => wp_kses_post( 'Type region code here. For more information visit - 
                    <a 
                    href="https://developers.google.com/chart/interactive/docs/gallery/geochart#continent-hierarchy-and-codes" 
                    target="_blank">Continent Hierarchy and Codes</a>.'
                ),
            ]
        );

        $this->add_control(
            'map_resolution',
            [
                'label' => esc_html__( 'Resolution', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'countries',
                'options' => [
                    'countries'  => esc_html__( 'Countries', 'selection' ),
                    'provinces' => esc_html__( 'Provinces', 'selection' ),
                    'metros' => esc_html__( 'Metros', 'selection' ),
                ],
                'description' => esc_html__( 'Provinces - Supported only for country regions and US state regions. 
                Not supported for all countries; please test a country to see whether this option is supported. 
                Metros - Supported for the US country region and US state regions only.', 'selection' ),
            ]
        );

        $this->add_control(
            'display_mode',
            [
                'label' => esc_html__( 'Display mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'regions',
                'options' => [
                    'regions'  => esc_html__( 'Regions', 'selection' ),
                    'markers' => esc_html__( 'Markers', 'selection' ),
                    'text' => esc_html__( 'Text', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'markers_magnifying_glass_zoom_factor',
            [
                'label' => esc_html__( 'Magnifying glass zoom factor', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'display_mode' => 'markers',
                ]
            ]
        );

        $this->add_control(
            'display_type',
            [
                'label' => esc_html__( 'Display type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'custom',
                'options' => [
                    'data-visual'  => esc_html__( 'Change colors depending on values', 'selection' ),
                    'custom' => esc_html__( 'Custom', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'tooltip_trigger',
            [
                'label' => esc_html__( 'Tooltip trigger', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'focus',
                'options' => [
                    'focus'  => esc_html__( 'Focus', 'selection' ),
                    'selection' => esc_html__( 'Selection', 'selection' ),
                    'none' => esc_html__( 'None', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'inactive_regions_color',
            [
                'label' => esc_html__( 'Inactive regions color', 'selection' ),
                'type' => Controls_Manager::COLOR,
            ]
        );

        $this->add_control(
            'map_background_color',
            [
                'label' => esc_html__( 'Background color', 'selection' ),
                'type' => Controls_Manager::COLOR,
            ]
        );

        $this->add_control(
            'borders_color',
            [
                'label' => esc_html__( 'Borders color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} path" => 'stroke: {{VALUE}} !important;',
                ],
            ]
        );

        $this->add_control(
            'enable_overlay',
            [
                'label' => esc_html__( 'Overlay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Markers Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_markers() {

        $this->start_controls_section( 'section_content_markers', [
            'label' => esc_html__( 'Markers', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'display_mode' => 'markers'
            ]
        ] );

        $this->add_control(
            'markers_enable_legend',
            [
                'label' => esc_html__( 'Enable legend', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'markers_legend_text_color',
            [
                'label' => esc_html__( 'Legend text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'condition' => [
                    'display_type' => 'data-visual',
                    'markers_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'markers_legend_font_size',
            [
                'label' => esc_html__( 'Legend text font size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual',
                    'markers_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'markers_legend_text_bold',
            [
                'label' => esc_html__( 'Legend text bold', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'markers_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'markers_legend_text_italic',
            [
                'label' => esc_html__( 'Legend text italic', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'markers_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'data_visualization_min_size',
            [
                'label' => esc_html__( 'Min size axis value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 1,
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'data_visualization_max_size',
            [
                'label' => esc_html__( 'Max size axis value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'data_marker_value_title',
            [
                'label' => esc_html__( 'Value title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type value title here', 'selection' ),
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'data_marker_additional_value_title',
            [
                'label' => esc_html__( 'Additional value title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type value title here', 'selection' ),
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $repeater_color_axis = new Repeater();

        $repeater_color_axis->add_control(
            'axis_color',
            [
                'label' => esc_html__( 'Color axis', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'description' => esc_html__( 'Specifies the range of colors for your data values. 
                The first element is the color given to the smallest value in your dataset, and the last element is
                the color given to the largest value in your dataset.', 'selection' ),
            ]
        );

        $this->add_control(
            'list_markers_color_axis',
            [
                'label' => esc_html__( 'Markers color axis', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_color_axis->get_controls(),
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ axis_color }}}',
            ]
        );

        $repeater_data_vis = new Repeater();

        $repeater_data_vis->add_control(
            'enable_markers_link',
            [
                'label' => esc_html__( 'Enable link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater_data_vis->add_control(
            'markers_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                    'custom_attributes' => '',
                ],
                'condition' => [
                    'enable_markers_link' => 'yes'
                ]
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_marker_region_name',
            [
                'label' => esc_html__( 'Region full name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type region name here', 'selection' ),
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_value',
            [
                'label' => esc_html__( 'Value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_additional_value',
            [
                'label' => esc_html__( 'Additional value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
            ]
        );

        $this->add_control(
            'list_markers_data_visual',
            [
                'label' => esc_html__( 'Markers values', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_data_vis->get_controls(),
                'default' => [
                    [
                        'data_visualization_marker_region_name' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ data_visualization_marker_region_name }}}',
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'enable_markers_link',
            [
                'label' => esc_html__( 'Enable link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater->add_control(
            'markers_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                    'custom_attributes' => '',
                ],
                'condition' => [
                    'enable_markers_link' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'custom_icon_type',
            [
                'label' => esc_html__( 'Icon type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'default',
                'options' => [
                    'icon'  => esc_html__( 'Icon', 'selection' ),
                    'image' => esc_html__( 'Image', 'selection' ),
                    'default' => esc_html__( 'Default', 'selection' ),
                ],
            ]
        );

        $repeater->add_control(
            'marker_image',
            [
                'label' => esc_html__( 'Choose image', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'condition' => [
                    'custom_icon_type' => 'image'
                ]
            ]
        );

        $repeater->add_control(
            'marker_icon',
            [
                'label' => esc_html__( 'Choose icon', 'mapper-name' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    'custom_icon_type' => 'icon'
                ]
            ]
        );


        $repeater->add_control(
            'marker_size',
            [
                'label' => esc_html__( 'Marker size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 10,
                ],
            ]
        );

        $repeater->add_control(
            'marker_color',
            [
                'label' => esc_html__( 'Marker color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'custom_icon_type',
                            'operator' => '==',
                            'value' => 'default'
                        ],
                        [
                            'name' => 'custom_icon_type',
                            'operator' => '==',
                            'value' => 'icon'
                        ]
                    ]
                ]
            ]
        );


        $repeater->add_control(
            'marker_latitude',
            [
                'label' => esc_html__( 'Latitude', 'selection' ),
                'label_block' => 'true',
                'default' => '-34.391',
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
            ]
        );

        $repeater->add_control(
            'marker_longitude',
            [
                'label' => esc_html__( 'Longitude', 'selection' ),
                'label_block' => 'true',
                'default' => '150.644',
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
            ]
        );

        $repeater->add_control(
            'marker_title',
            [
                'label' => esc_html__( 'Title', 'selection' ),
                'label_block' => 'true',
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
            ]
        );

        $repeater->add_control(
            'marker_tooltip_content',
            [
                'label' => esc_html__( 'Tooltip content', 'selection' ),
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::WYSIWYG,
                'placeholder' => esc_html__( 'Type tooltip content here', 'selection' ),
            ]
        );

        $this->add_control(
            'list_markers',
            [
                'label' => esc_html__( 'Markers', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'marker_title' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'title_field' => '{{{ marker_title }}}',
                'condition' => [
                    'display_type' => 'custom'
                ],
            ]
        );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Regions Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_regions() {

        $this->start_controls_section( 'section_content_regions', [
            'label' => esc_html__( 'Regions', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'display_mode' => 'regions'
            ]
        ] );

        $this->add_control(
            'regions_enable_legend',
            [
                'label' => esc_html__( 'Enable legend', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'regions_legend_text_color',
            [
                'label' => esc_html__( 'Legend text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'condition' => [
                    'display_type' => 'data-visual',
                    'regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'regions_legend_font_size',
            [
                'label' => esc_html__( 'Legend text font size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual',
                    'regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'regions_legend_text_bold',
            [
                'label' => esc_html__( 'Legend text bold', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'regions_legend_text_italic',
            [
                'label' => esc_html__( 'Legend text italic', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'data_region_value_title',
            [
                'label' => esc_html__( 'Value title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type value title here', 'selection' ),
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'data_region_additional_value_title',
            [
                'label' => esc_html__( 'Additional value title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type value title here', 'selection' ),
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $repeater_color_axis = new Repeater();

        $repeater_color_axis->add_control(
            'axis_color',
            [
                'label' => esc_html__( 'Color axis', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'description' => esc_html__( 'Specifies the range of colors for your data values. 
                The first element is the color given to the smallest value in your dataset, and the last element is
                the color given to the largest value in your dataset.', 'selection' ),
            ]
        );

        $this->add_control(
            'list_regions_color_axis',
            [
                'label' => esc_html__( 'Regions color axis', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_color_axis->get_controls(),
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ axis_color }}}',
            ]
        );


        $repeater_data_vis = new Repeater();

        $repeater_data_vis->add_control(
            'enable_regions_link',
            [
                'label' => esc_html__( 'Enable link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater_data_vis->add_control(
            'regions_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                    'custom_attributes' => '',
                ],
                'condition' => [
                    'enable_regions_link' => 'yes'
                ]
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_region_name',
            [
                'label' => esc_html__( 'Region name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Australia', 'selection' ),
                'placeholder' => esc_html__( 'Type region name here', 'selection' ),
            ]
        );


        $repeater_data_vis->add_control(
            'data_visualization_value',
            [
                'label' => esc_html__( 'Value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_additional_value',
            [
                'label' => esc_html__( 'Additional value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
            ]
        );

        $this->add_control(
            'list_regions_data_visual',
            [
                'label' => esc_html__( 'Regions values', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_data_vis->get_controls(),
                'default' => [
                    [
                        'data_visualization_region_name' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ data_visualization_region_name }}}',
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'enable_regions_link',
            [
                'label' => esc_html__( 'Enable link', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater->add_control(
            'regions_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                    'custom_attributes' => '',
                ],
                'condition' => [
                    'enable_regions_link' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'active_region_color',
            [
                'label' => esc_html__( 'Active region color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
            ]
        );

        $repeater->add_control(
            'active_region_name',
            [
                'label' => esc_html__( 'Name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Australia', 'selection' ),
                'placeholder' => esc_html__( 'Type name here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'region_code_name',
            [
                'label' => esc_html__( 'Region code', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type region code here', 'selection' ),
                'default' => esc_html__( 'AU', 'selection' ),
                'description' => wp_kses_post( ' 
                    <a 
                    href="https://developers.google.com/chart/interactive/docs/gallery/geochart#continent-hierarchy-and-codes" 
                    target="_blank">Continent Hierarchy and Codes</a>.'
                ),
            ]
        );

        $repeater->add_control(
            'region_tooltip_content',
            [
                'label' => esc_html__( 'Tooltip content', 'selection' ),
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::WYSIWYG,
                'placeholder' => esc_html__( 'Type tooltip content here', 'selection' ),
            ]
        );

        $this->add_control(
            'list_regions',
            [
                'label' => esc_html__( 'Regions', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'active_region_name' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'condition' => [
                    'display_type' => 'custom'
                ],
                'title_field' => '{{{ active_region_name }}}',
            ]
        );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Text Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_text() {

        $this->start_controls_section( 'section_content_text', [
            'label' => esc_html__( 'Text', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'display_mode' => 'text'
            ]
        ] );

        $this->add_control(
            'text_regions_enable_legend',
            [
                'label' => esc_html__( 'Enable legend', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'display_type' => 'data-visual'
                ]
            ]
        );

        $this->add_control(
            'text_legend_text_color',
            [
                'label' => esc_html__( 'Legend text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'condition' => [
                    'display_type' => 'data-visual',
                    'text_regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'text_legend_font_size',
            [
                'label' => esc_html__( 'Legend text font size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual',
                    'text_regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'text_legend_text_bold',
            [
                'label' => esc_html__( 'Legend text bold', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'text_regions_enable_legend' => 'yes',
                ]
            ]
        );

        $this->add_control(
            'text_legend_text_italic',
            [
                'label' => esc_html__( 'Legend text italic', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
                'condition' => [
                    'display_type' => 'data-visual',
                    'text_regions_enable_legend' => 'yes',
                ]
            ]
        );

        $repeater_color_axis = new Repeater();

        $repeater_color_axis->add_control(
            'axis_color',
            [
                'label' => esc_html__( 'Color axis', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
                'description' => esc_html__( 'Specifies the range of colors for your data values. 
                The first element is the color given to the smallest value in your dataset, and the last element is
                the color given to the largest value in your dataset.', 'selection' ),
            ]
        );

        $this->add_control(
            'list_regions_text_color_axis',
            [
                'label' => esc_html__( 'Regions text color axis', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_color_axis->get_controls(),
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ axis_color }}}',
            ]
        );

        $repeater_data_vis = new Repeater();

        $repeater_data_vis->add_control(
            'data_visualization_region_name',
            [
                'label' => esc_html__( 'Region name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type region name here', 'selection' ),
            ]
        );

        $repeater_data_vis->add_control(
            'data_visualization_value',
            [
                'label' => esc_html__( 'Value', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'step' => 1,
                'default' => 100,
            ]
        );

        $this->add_control(
            'list_regions_text_data_visual',
            [
                'label' => esc_html__( 'Regions text values', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater_data_vis->get_controls(),
                'default' => [
                    [
                        'data_visualization_region_name' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'condition' => [
                    'display_type' => 'data-visual'
                ],
                'title_field' => '{{{ data_visualization_region_name }}}',
            ]
        );


        $repeater = new Repeater();

        $repeater->add_control(
            'active_region_text_color',
            [
                'label' => esc_html__( 'Active region text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'alpha' => false,
            ]
        );

        $repeater->add_control(
            'active_region_text',
            [
                'label' => esc_html__( 'Region full name', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Australia', 'selection' ),
                'placeholder' => esc_html__( 'Type name here', 'selection' ),
            ]
        );

        $this->add_control(
            'list_text_regions',
            [
                'label' => esc_html__( 'Regions', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'active_region_text' => esc_html__( 'Australia', 'selection' ),
                    ],
                ],
                'condition' => [
                    'display_type' => 'custom'
                ],
                'title_field' => '{{{ active_region_text }}}',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Method for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {


        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param array $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ? $opts['html_class'] : '',
            'additional_html_class' => array_key_exists( 'additional_html_class', $opts ) ?
                $opts['additional_html_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ? $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ? $opts['include_bg'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ? $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ? $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ? $opts['color_hover_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'include_css_filters' => array_key_exists( 'include_css_filters', $opts ) ?
                $opts['include_css_filters'] : false,
            'typography_class' => array_key_exists( 'typography_class', $opts ) ? $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] :
                Color::COLOR_3,
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
                ]
            );

        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'],
                ]
            );
        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].$style_opts['additional_html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }


        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
                ]
            );
        }


        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover' . $section_id,
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'].':hover',
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'].$style_opts['additional_html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover".$style_opts['additional_html_class'].":hover",
            ]
        );

        $this->end_controls_tab();


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Map.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_map() {

        $this->start_controls_section( 'section_style_map', [
            'label' => esc_html__( 'Map', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'map_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ]
            ]
        );

        $this->add_responsive_control(
            'map_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'text_mode_typography',
                'label' => esc_html__('Text typography', 'selection'),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} text",
                'condition' => [
                    'display_mode' => 'text'
                ]
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_map', [
            'html_class' => 'mdp-mapper-elementor-box svg',
            'include_color' => false,
            'include_bg' => false,
            'include_css_filters' => true,
            'include_typography' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Tooltip.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_tooltip() {

        $this->start_controls_section( 'section_style_tooltip', [
            'label' => esc_html__( 'Tooltip', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_mode!' => 'text',
            ]
        ] );


        $this->generate_margin_padding_controls(
            'section_style_tooltip',
            'google-visualization-tooltip-item:first-child'
        );


        $this->add_responsive_control(
            'tooltip_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .google-visualization-tooltip' => 'height: {{SIZE}}{{UNIT}} !important;'
                ]
            ]
        );

        $this->add_responsive_control(
            'tooltip_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .google-visualization-tooltip' => 'width: {{SIZE}}{{UNIT}} !important;'
                ]
            ]
        );

        $this->add_control(
            'tooltip_font',
            [
                'label' => esc_html__( 'Tooltip font', 'selection' ),
                'type' => Controls_Manager::FONT,
                'selectors' => [
                    '{{WRAPPER}} .google-visualization-tooltip-item span, {{WRAPPER}} .google-visualization-tooltip-item' => 'font-family: {{VALUE}} !important;'
                ]
            ]
        );

        $this->add_control(
            'tooltip_font_size',
            [
                'label' => esc_html__( 'Tooltip font size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
            ]
        );

        $this->add_control(
            'tooltip_text_bold',
            [
                'label' => esc_html__( 'Tooltip text bold', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $this->add_control(
            'tooltip_text_italic',
            [
                'label' => esc_html__( 'Tooltip text italic', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_tooltip', [
            'html_class' => 'google-visualization-tooltip',
            'include_color' => false,
            'include_typography' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Tooltip Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_tooltip_title() {

        $this->start_controls_section( 'section_style_tooltip_title', [
            'label' => esc_html__( 'Tooltip title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_mode!' => 'text',
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_tooltip_title',
            'google-visualization-tooltip-item:first-child'
        );


        $this->add_control(
            'tooltip_title_alignment',
            [
                'label' => esc_html__( 'Title align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .google-visualization-tooltip-item:first-child' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_tooltip_title', [
            'html_class' => 'google-visualization-tooltip-item:first-child',
            'color_hover_class' => 'google-visualization-tooltip-item:first-child:hover span',
            'include_typography' => false,
            'color_class' => 'google-visualization-tooltip-item:first-child span',
            'additional_transition_selector' => '{{WRAPPER}} .google-visualization-tooltip-item:first-child span'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Value.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_data_vis_value() {
        $this->start_controls_section( 'section_style_data_vis_value', [
            'label' => esc_html__( 'Value', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_mode!' => 'text',
                'display_type' => 'data-visual'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_data_vis_value',
            'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2)'
        );

        $this->add_control(
            'section_style_data_vis_value_alignment',
            [
                'label' => esc_html__( 'Description align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2)' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_data_vis_value', [
            'html_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2)',
            'color_hover_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2):hover span',
            'include_typography' => false,
            'color_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2) span',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:nth-child(2) span'
        ] );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Additional Value.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_data_vis_additional_value() {
        $this->start_controls_section( 'section_style_data_vis_additional_value', [
            'label' => esc_html__( 'Additional value', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_mode!' => 'text',
                'display_type' => 'data-visual'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_data_vis_additional_value',
            'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child'
        );

        $this->add_control(
            'section_style_data_vis_additional_value_alignment',
            [
                'label' => esc_html__( 'Description align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_data_vis_additional_value', [
            'html_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child',
            'color_hover_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child:hover span',
            'include_typography' => false,
            'color_class' => 'mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child span',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-mapper-elementor-data-vis-display-type .google-visualization-tooltip-item:last-child span'
        ] );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Style Tooltip Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_tooltip_description() {

        $this->start_controls_section( 'section_style_tooltip_description', [
            'label' => esc_html__( 'Tooltip description', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'display_mode!' => 'text',
                'display_type' => 'custom'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_tooltip_description',
            'google-visualization-tooltip-item:last-child'
        );


        $this->add_control(
            'tooltip_description_alignment',
            [
                'label' => esc_html__( 'Description align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-mapper-elementor-custom-display-type .google-visualization-tooltip-item:last-child' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_tooltip_description', [
            'html_class' => 'mdp-mapper-elementor-custom-display-type .google-visualization-tooltip-item:last-child',
            'color_hover_class' => 'mdp-mapper-elementor-custom-display-type .google-visualization-tooltip-item:last-child:hover span',
            'include_typography' => false,
            'color_class' => 'mdp-mapper-elementor-custom-display-type .google-visualization-tooltip-item:last-child span',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-mapper-elementor-custom-display-type .google-visualization-tooltip-item:last-child span'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Overlay.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_overlay() {

        $this->start_controls_section('section_style_overlay', [
            'label' => esc_html__( 'Overlay', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE,
            'condition' => [
                'enable_overlay' => 'yes'
            ]
        ] );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'overlay_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-mapper-elementor-overlay',
            ]
        );

        $this->add_control(
            'overlay_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    '' => esc_html__( 'Normal', 'selection' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-mapper-elementor-overlay' => 'mix-blend-mode: {{VALUE}}',
                ],
                'separator' => 'none',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Creates data attributes for regions display mode.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_regions_data_attrs( $settings ) {
        $regions = json_encode( $settings['list_regions'] );
        $regions_data_vis = json_encode( $settings['list_regions_data_visual'] );
        $regions_color_axis = json_encode( $settings['list_regions_color_axis'] );

        return sprintf(
            ' data-regions="%s" 
                    data-region-color-axis="%s" 
                    data-region-enable-legend="%s"
                    data-region-legend-color="%s"
                    data-region-legend-font-size="%s"
                    data-region-legend-bold-text="%s"
                    data-region-legend-italic-text="%s"
                    data-region-value-title="%s"
                    data-region-additional-value-title="%s"',
    $settings['display_type'] === 'custom' ? esc_attr( $regions ) : esc_attr( $regions_data_vis ),
            esc_attr( $regions_color_axis ),
            esc_attr( $settings['regions_enable_legend'] ),
            $settings['regions_enable_legend'] === 'yes' ? esc_attr( $settings['regions_legend_text_color'] ) : '',
            $settings['regions_enable_legend'] === 'yes' ?
                esc_attr( $settings['regions_legend_font_size']['size'] ) :
                '',
            $settings['regions_enable_legend'] === 'yes' ? esc_attr( $settings['regions_legend_text_bold'] ) : '',
            $settings['regions_enable_legend'] === 'yes' ? esc_attr( $settings['regions_legend_text_italic'] ) : '',
            esc_attr( $settings['data_region_value_title'] ),
            esc_attr( $settings['data_region_additional_value_title'] )
        );
    }

    /**
     * Creates data attributes for markers display mode.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_markers_data_attrs( $settings ) {
        $markers_color_axis = json_encode( $settings['list_markers_color_axis'] );
        $markers_data_vis = json_encode( $settings['list_markers_data_visual'] );
        $markers = json_encode( $settings['list_markers'] );


        return sprintf(
            ' data-markers="%s"
            data-markers-color-axis="%s"
            data-markers-enable-legend="%s"
            data-markers-legend-color="%s"
            data-markers-legend-font-size="%s"
            data-markers-legend-bold-text="%s"
            data-markers-legend-italic-text="%s"
            data-marker-value-title="%s"
            data-marker-additional-value-title="%s"
            data-marker-size-axis-min-val="%s"
            data-marker-size-axis-max-val="%s"',
            $settings['display_type'] === 'custom' ? esc_attr( $markers ) : esc_attr( $markers_data_vis ),
            esc_attr( $markers_color_axis ),
            esc_attr( $settings['markers_enable_legend'] ),
            $settings['markers_enable_legend'] === 'yes' ? esc_attr( $settings['markers_legend_text_color'] ) : '',
            $settings['markers_enable_legend'] === 'yes' ? esc_attr( $settings['markers_legend_font_size']['size'] ) : '',
            $settings['markers_enable_legend'] === 'yes' ? esc_attr( $settings['markers_legend_text_bold'] ) : '',
            $settings['markers_enable_legend'] === 'yes' ? esc_attr( $settings['markers_legend_text_italic'] ) : '',
            esc_attr( $settings['data_marker_value_title'] ),
            esc_attr( $settings['data_marker_additional_value_title'] ),
            esc_attr( $settings['data_visualization_min_size'] ),
            esc_attr( $settings['data_visualization_max_size'] )
        );
    }

    /**
     * Creates data attributes for text display mode.
     *
     * @param $settings
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function create_text_regions_data_attrs( $settings ) {
        $regions_text_data_vis = json_encode( $settings['list_regions_text_data_visual'] );
        $regions_text_color_axis = json_encode( $settings['list_regions_text_color_axis'] );
        $text_regions = json_encode( $settings['list_text_regions'] );

        return sprintf(
            'data-text-regions="%s"
                    data-text-color-axis="%s"
                    data-text-regions-enable-legend="%s"
                    data-text-legend-color="%s"
                    data-text-legend-font-size="%s"
                    data-text-legend-bold-text="%s"
                    data-text-legend-italic-text="%s"',

            $settings['display_type'] === 'custom' ?
                esc_attr( $text_regions ) :
                esc_attr( $regions_text_data_vis ),
            esc_attr( $regions_text_color_axis ),
            esc_attr( $settings['text_regions_enable_legend'] ),
            $settings['text_regions_enable_legend'] === 'yes' ?
                esc_attr( $settings['text_legend_text_color'] ) :
                '',
            $settings['text_regions_enable_legend'] === 'yes' ?
                esc_attr( $settings['text_legend_font_size']['size'] ) :
                '',
            $settings['text_regions_enable_legend'] === 'yes' ?
                esc_attr( $settings['text_legend_text_bold'] ) :
                '',
            $settings['text_regions_enable_legend'] === 'yes' ?
                esc_attr( $settings['text_legend_text_italic'] ) :
                ''
        );
    }

    /**
     * Converts string to camel case.
     *
     * @param $string
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function to_camel_case( $string ) {

        $str = str_replace( ' ', '', ucwords( str_replace( '_', ' ', $string ) ) );

        $str[0] = strtolower( $str[0] );

        return $str;
    }

    /**
     * Get settings by name. Returns settings array based on control prefix.
     *
     * @param $settings
     * @param $setting_name
     * @return array
     * @since 1.0.0
     * @access private
     */
    private function get_settings_by_name( $setting_name, $settings ) {
        $player_settings = [];

        /** Set settings by feature name. */
        foreach ( $settings as $key => $setting ) {
            if ( strpos( $key, $setting_name ) === 0 ) {
                $key_parts = explode( '_', $key, 2 );
                if ( !isset( $setting ) || ( is_array( $setting ) && empty( $setting['size'] ) ) ) { continue; }
                $player_settings[$this->to_camel_case( $key_parts[1] )] = is_array( $setting ) ?
                    $setting['size'] : $setting;
            }
        }

        return $player_settings;

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();

        $regions_data_attrs = $this->create_regions_data_attrs( $settings );
        $markers_data_attrs = $this->create_markers_data_attrs( $settings );
        $text_regions_data_attr = $this->create_text_regions_data_attrs( $settings );
        $display_custom_type_class = $settings['display_type'] === 'custom' ?
            'mdp-mapper-elementor-custom-display-type' :
            'mdp-mapper-elementor-data-vis-display-type';
        $overlay = '<div class="mdp-mapper-elementor-overlay"></div>';
        $map_width = json_encode( $this->get_settings_by_name( 'map_width', $settings ) );
        $map_height = json_encode( $this->get_settings_by_name( 'map_height', $settings ) );

        echo sprintf(
                '<!-- Start Mapper for Elementor WordPress Plugin -->
                        <div class="mdp-mapper-elementor-box %s"
                        data-resolution="%s"
                        data-tooltip-trigger="%s"
                        data-tooltip-font-size="%s"
                        data-tooltip-text-bold="%s"
                        data-tooltip-text-italic="%s"
                        data-display-type="%s"
                        data-map-height="%s"
                        data-map-width="%s"
                        data-display-mode="%s" 
                        data-inactive-region-color="%s"
                        data-background-color="%s"
                        data-region-to-display="%s" 
                        data-magnifying-glass-zoom="%s"
                        %s 
                        %s 
                        %s >
                            <div class="mdp-mapper-elementor-map-wrapper"></div>
                            %s
                        </div>
                        <!-- End Mapper for Elementor WordPress Plugin -->',
                 esc_attr( $display_custom_type_class ),
                        esc_attr( $settings['map_resolution'] ),
                        esc_attr( $settings['tooltip_trigger'] ),
                        isset( $settings['tooltip_font_size'] ) ?
                            esc_attr( $settings['tooltip_font_size']['size'] ) :
                            '',
                        esc_attr( $settings['tooltip_text_bold'] ),
                        esc_attr( $settings['tooltip_text_italic'] ),
                        esc_attr( $settings['display_type'] ),
                        esc_attr( $map_height ),
                        esc_attr( $map_width ),
                        esc_attr( $settings['display_mode'] ),
                        esc_attr( $settings['inactive_regions_color'] ),
                        esc_attr( $settings['map_background_color'] ),
                        esc_attr( $settings['region_to_display'] ),
                        isset( $settings['markers_magnifying_glass_zoom_factor'] ) ?
                            esc_attr( $settings['markers_magnifying_glass_zoom_factor']['size'] ) :
                            '',
                        $settings['display_mode'] === 'regions' ? $regions_data_attrs : '',
                        $settings['display_mode'] === 'markers' ? $markers_data_attrs : '',
                        $settings['display_mode'] === 'text' ? $text_regions_data_attr : '',
                        $settings['enable_overlay'] === 'yes' ? $overlay : ''
        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/mapper';

    }

}
