<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Producter for Elementor
 * Customizable product fields for Elementor editor
 * Exclusively on https://1.envato.market/producter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Product Images - Custom Elementor Widget.
 **/
class product_images_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-producter-elementor', UnityPlugin::get_url() . 'css/producter-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );


        wp_register_script( 'mdp-producter-elementor', UnityPlugin::get_url() . 'js/producter-elementor' . UnityPlugin::get_suffix() . '.js', [ 'wc-single-product', 'flexslider', 'zoom' ], UnityPlugin::get_version(), true );


    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-producter-product-images-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Product Images', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-product-images-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Producter', 'woocommerce' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-producter-elementor', 'mdp-selection-elementor-admin', 'elementor-icons-fa-solid' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-producter-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Product Images Content Section. */
        $this->section_content_product_images();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Product Image. */
        $this->section_style_product_image();

        /** Style -> Section Style Product Thumbnails. */
        $this->section_style_product_image_thumbnails();

        /** Style -> Section Style Product Image Sale Flash. */
        $this->section_style_product_image_sale_flash();

    }

    /**
     * Add widget controls: Content -> Product Images Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_product_images() {

        $this->start_controls_section( 'section_content_product_images', [
            'label' => esc_html__( 'Product Images', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        if ( Caster::get_instance()->is_active_woocommerce() ) {
            $products_list = [];

            $products = wc_get_products( ['return' => 'ids', 'limit' => -1] );
            foreach ( $products as $product ) {
                $products_list[$product] = get_the_title( $product );
            }

            if ( empty( wc_get_product() ) ) {

                $this->add_control(
                    'products',
                    [
                        'label' => esc_html__( 'Products', 'selection' ),
                        'type' => Controls_Manager::SELECT2,
                        'multiple' => false,
                        'options' => $products_list,
                        'default' => !empty( $products_list ) ? array_keys( $products_list )[0] : ''
                    ]
                );

            }
        }

        $this->add_control(
            'show_sale_flash',
            [
                'label' => esc_html__( 'Sale flash', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_control(
            $section_id.'_margin_padding_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param array $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ? $opts['html_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ? $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ? $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ? $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ? $opts['include_transition'] : true,
            'include_box_shadow' => array_key_exists( 'include_box_shadow', $opts ) ? $opts['include_box_shadow'] : true,
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ? $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ? $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ? $opts['color_hover_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ? $opts['color_hover_selector'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ? $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ? $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ? $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ? $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        if ( $style_opts['include_box_shadow'] ) {

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_normal',
                    'label' => esc_html__('Box Shadow', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'],
                ]
            );

        }


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }


        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        if ( $style_opts['include_box_shadow'] ) {

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_hover',
                    'label' => esc_html__('Box Shadow', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'] . ":hover",
                ]
            );

        }

        $this->end_controls_tab();


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Images.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_image() {

        $this->start_controls_section( 'section_style_product', [
            'label' => esc_html__( 'Product image', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .woocommerce-product-gallery__image img" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product', [
            'html_class' => 'woocommerce-product-gallery__image img',
            'include_color' => false,
            'include_typography' => false,
            'include_box_shadow' => false
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Image Thumbnails.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_image_thumbnails() {

        $this->start_controls_section( 'section_style_product_image_thumbnails', [
            'label' => esc_html__( 'Thumbnails', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_product_image_thumbnails', 'flex-control-thumbs li' );

        $this->generate_typography_tabs_controls( 'section_style_product_image_thumbnails', [
            'html_class' => 'flex-control-thumbs li',
            'include_color' => false,
            'include_typography' => false,
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Image Sale Flash.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_image_sale_flash() {
        $this->start_controls_section( 'section_style_product_image_sale_flash', [
            'label' => esc_html__( 'Sale flash', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                    'show_sale_flash' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_style_product_image_sale_flash', 'onsale' );

        $this->generate_typography_tabs_controls( 'section_style_product_image_sale_flash', [
            'html_class' => 'onsale',
            'include_color' => true,
            'color_class' => 'onsale',
            'color_hover_class' => 'onsale:hover',
            'include_typography' => false,
        ] );

        $this->end_controls_section();
    }



    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Install WooCommerce to display product images!' );
            return;
        }
        global $product;
        $settings = $this->get_settings_for_display();
        $product = wc_get_product();

        if ( empty( $product ) ) {
            $product = wc_get_product( esc_sql( $settings['products'] ) );
            if ( empty( $product ) || $product->get_status() === 'trash'  ) {
                return;
            }
        }

        ?>
        <!-- Start Producter for Elementor WordPress Plugin -->
        <div class="mdp-producter-elementor-product-images-box">
            <div class="mdp-producter-elementor-product-wrapper">
                <?php if ( $settings['show_sale_flash'] === 'yes' ) {
                    wc_get_template( 'loop/sale-flash.php' );
                }
                wc_get_template( 'single-product/product-image.php' );
                ?>
            </div>
        </div>
        <!-- End Producter for Elementor WordPress Plugin -->
	    <?php
        // on render widget from editor
        if ( wp_doing_ajax() ) {
            ?>
            <script>
                mdpProducter.producterImages();
            </script>
            <?php
        }
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/producter';

    }

}
