<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Producter for Elementor
 * Customizable product fields for Elementor editor
 * Exclusively on https://1.envato.market/producter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Product Meta - Custom Elementor Widget.
 **/
class product_meta_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-producter-elementor', UnityPlugin::get_url() . 'css/producter-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-producter-product-meta-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Product Meta', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-product-meta-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Producter', 'woocommerce' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-producter-elementor', 'mdp-selection-elementor-admin', 'elementor-icons-fa-solid' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-producter' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Product Meta Content Section. */
        $this->section_content_product_meta();

        /** Content -> Product Meta Captions Content Section. */
        $this->section_content_product_meta_captions();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Product Meta. */
        $this->section_style_product_meta();

        /** Style -> Section Style Product Meta Captions. */
        $this->section_style_product_meta_captions();

        /** Style -> Section Style Product Meta Separator. */
        $this->section_style_product_meta_separator();

    }

    /**
     * Add widget controls: Content -> Product Meta Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_product_meta() {

        $this->start_controls_section( 'section_content_product_meta', [
            'label' => esc_html__( 'Product Meta', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );


        if ( Caster::get_instance()->is_active_woocommerce() ) {
            $products_list = [];

            $products = wc_get_products( ['return' => 'ids', 'limit' => -1] );
            foreach ( $products as $product ) {
                $products_list[$product] = get_the_title( $product );
            }

            if ( empty( wc_get_product() ) ) {

                $this->add_control(
                    'products',
                    [
                        'label' => esc_html__( 'Products', 'selection' ),
                        'type' => Controls_Manager::SELECT2,
                        'multiple' => false,
                        'options' => $products_list,
                        'default' => !empty( $products_list ) ? array_keys( $products_list )[0] : ''
                    ]
                );
            }
        }

        $this->add_control(
            'product_meta_view',
            [
                'label'     => esc_html__( 'View', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'table'    => esc_html__( 'Table', 'selection' ),
                    'inline'    => esc_html__( 'Inline', 'selection' ),
                ],
                'default'   => 'inline',
            ]
        );

        $this->add_responsive_control(
            'product_meta_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-producter-elementor-product-wrapper .product_meta' => 'justify-content: {{VALUE}}',
                        '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-table .product_meta' => 'align-items: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );


        $this->add_control(
            'product_meta_separator',
            [
                'label' => esc_html__( 'Separator', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'product_meta_separator_type',
            [
                'label'     => esc_html__( 'Separator Type', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'text'    => esc_html__( 'Text', 'selection' ),
                    'icon'    => esc_html__( 'Icon', 'selection' ),
                ],
                'condition' => [ 'product_meta_separator' => 'yes' ],
                'default'   => 'text',
            ]
        );

        $this->add_control(
            'product_meta_separator_text',
            [
                'label' => esc_html__( 'Separator Text', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
                'condition' => [
                    'product_meta_separator' => 'yes',
                    'product_meta_separator_type' => 'text'
                ]
            ]
        );


        $this->add_control(
            'product_meta_separator_icon',
            [
                'label' => esc_html__( 'Separator Icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    'product_meta_separator' => 'yes',
                    'product_meta_separator_type' => 'icon'
                ]
            ]
        );


        $this->end_controls_section();
    }

    /**
     * Add widget controls: Content -> Product Meta Captions Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_product_meta_captions() {

        $this->start_controls_section( 'section_content_product_meta_captions', [
            'label' => esc_html__( 'Captions', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'product_meta_tag_single',
            [
                'label' => esc_html__( 'Tag Single', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->add_control(
            'product_meta_tag_plural',
            [
                'label' => esc_html__( 'Tag Plural', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->add_control(
            'product_meta_category_single',
            [
                'label' => esc_html__( 'Category Single', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'separator' => 'before',
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->add_control(
            'product_meta_category_plural',
            [
                'label' => esc_html__( 'Category Plural', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->add_control(
            'product_meta_sku',
            [
                'label' => esc_html__( 'SKU', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'separator' => 'before',
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->add_control(
            'product_meta_missing',
            [
                'label' => esc_html__( 'Missing', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'label_block' => true,
                'placeholder' => esc_html__( 'Type your text here', 'selection' ),
            ]
        );

        $this->end_controls_section();
    }



    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_control(
            $section_id.'_margin_padding_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param array $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ? $opts['html_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ? $opts['include_color'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ? $opts['additional_color'] : false,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ? $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ? $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ? $opts['include_transition'] : true,
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ? $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ? $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ? $opts['color_hover_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ? $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ? $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ? $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ? $opts['additional_color_hover_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ? $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ? $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ? $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ? $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $style_opts['include_color'] ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}};',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'crumber-elementor' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}};',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}};',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'crumber-elementor'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}};',
                    ],
                ]
            );
        }


        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Product Meta.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_meta() {

        $this->start_controls_section( 'section_style_product', [
            'label' => esc_html__( 'Product Meta', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_product', 'mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta .mdp-producter-elementor-product-meta-content' );

        $this->add_control(
            'product_meta_space_between',
            [
                'label' => esc_html__( 'Space Between', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-inline .mdp-producter-elementor-product-meta' => 'margin-right: {{SIZE}}{{UNIT}}; margin-left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-inline .mdp-producter-elementor-product-meta:first-child' => 'margin-left: 0;',
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-inline .mdp-producter-elementor-product-meta:last-of-type' => 'margin-right: 0;',
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-table .mdp-producter-elementor-product-meta' => 'margin-top: {{SIZE}}{{UNIT}}; margin-bottom: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-table .mdp-producter-elementor-product-meta:first-child' => 'margin-top: 0;',
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-view-table .mdp-producter-elementor-product-meta:last-of-type' => 'margin-bottom: 0;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product', [
            'html_class' => 'mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta .mdp-producter-elementor-product-meta-content',
            'include_color' => true,
            'color_class' => 'mdp-producter-elementor-product-meta, {{WRAPPER}} .mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta a',
            'color_hover_class' => 'mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta:hover, {{WRAPPER}} .mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta:hover a, {{WRAPPER}} .mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta:hover span',
            'include_typography' => true,
            'typography_class' => 'mdp-producter-elementor-product-meta',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-producter-elementor-product-meta a, {{WRAPPER}} .mdp-producter-elementor-product-meta span'
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Meta Captions.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_meta_captions() {

        $this->start_controls_section( 'section_style_product_meta_captions', [
            'label' => esc_html__( 'Product Meta Captions', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_product_meta_captions', 'mdp-producter-elementor-product-meta-caption' );

        $this->generate_typography_tabs_controls( 'section_style_product_meta_captions', [
            'html_class' => 'mdp-producter-elementor-product-meta-caption',
            'include_color' => true,
            'color_class' => 'mdp-producter-elementor-product-meta-caption',
            'color_hover_class' => 'mdp-producter-elementor-product-wrapper .mdp-producter-elementor-product-meta:hover .mdp-producter-elementor-product-meta-caption',
            'include_typography' => true,
            'typography_class' => 'mdp-producter-elementor-product-meta-caption',
        ] );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Meta Separator.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_meta_separator() {

        $this->start_controls_section( 'section_style_product_meta_separator', [
            'label' => esc_html__( 'Separator', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_product_meta_separator', 'mdp-producter-elementor-product-meta-separator' );

        $this->add_responsive_control(
            'product_meta_separator_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 200,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 16,
                ],
                'separator' => 'after',
                'condition' => [
                       'product_meta_separator_type' => 'icon'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-producter-elementor-product-meta-separator-icon' => 'font-size: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_meta_separator', [
            'html_class' => 'mdp-producter-elementor-product-meta-separator',
            'include_color' => true,
            'color_class' => 'mdp-producter-elementor-product-meta-separator span, {{WRAPPER}} .mdp-producter-elementor-product-meta-separator i',
            'color_hover_class' => 'mdp-producter-elementor-product-meta-separator:hover span, {{WRAPPER}} .mdp-producter-elementor-product-meta-separator:hover i',
            'include_typography' => true,
            'typography_class' => 'mdp-producter-elementor-product-meta-separator',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-producter-elementor-product-meta-separator span, {{WRAPPER}} .mdp-producter-elementor-product-meta-separator i'
        ] );

        $this->end_controls_section();

    }



    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Install WooCommerce to display product meta!' );
            return;
        }

        $settings = $this->get_settings_for_display();
        global $product;

        $product = wc_get_product();

        if ( empty( $product ) ) {
            $product = wc_get_product( esc_sql( $settings['products'] ) );
            if ( empty( $product ) || $product->get_status() === 'trash' ) {
                return;
            }
        }


        $sku = !empty( $settings['product_meta_missing'] ) ? $settings['product_meta_missing'] : esc_html__( 'N/A', 'selection' );
        $category_single = !empty( $settings['product_meta_category_single'] ) ? $settings['product_meta_category_single'] : esc_html__( 'Category:', 'selection' );
        $category_plural = !empty( $settings['product_meta_category_plural'] ) ? $settings['product_meta_category_plural'] : esc_html__( 'Categories:', 'selection' );
        $tag_single = !empty( $settings['product_meta_tag_single'] ) ? $settings['product_meta_tag_single'] : esc_html__( 'Tag:', 'selection' );
        $tag_plural = !empty( $settings['product_meta_tag_plural'] ) ? $settings['product_meta_tag_plural'] : esc_html__( 'Tags:', 'selection' );

        ?>
        <!-- Start Producter for Elementor WordPress Plugin -->
        <div class="mdp-producter-elementor-product-meta-box">
            <div class="mdp-producter-elementor-product-wrapper mdp-producter-elementor-product-meta-view-<?php esc_attr_e( $settings['product_meta_view'] ); ?>">
                <div class="product_meta">

                    <?php do_action( 'woocommerce_product_meta_start' ); ?>

                    <?php if ( ( $sku || $product->is_type( 'variable' ) ) && wc_product_sku_enabled() ): ?>
                        <span class="sku_wrapper mdp-producter-elementor-product-meta">
                            <span class="mdp-producter-elementor-product-meta-caption">
                                <?php esc_html_e( !empty( $settings['product_meta_sku'] ) ? $settings['product_meta_sku'] : 'SKU:' );  ?>
                            </span>
                            <span class="mdp-producter-elementor-product-meta-content">
                                <span class="sku">
                                    <?php esc_html_e( $product->get_sku() ? $product->get_sku() : $sku ); ?>
                                </span>
                            </span>
                        </span>
                        <?php if ( $settings['product_meta_separator'] === 'yes' ): ?>
                            <div class="mdp-producter-elementor-product-meta-separator">
                                <?php if ( $settings['product_meta_separator_type'] === 'text' ) : ?>
                                    <span><?php esc_html_e( $settings['product_meta_separator_text'] ); ?></span>
                                <?php else: ?>
                                    <div class="mdp-producter-elementor-product-meta-separator-icon">
                                        <?php Icons_Manager::render_icon( $settings['product_meta_separator_icon'], ['aria-hidden' => 'true'] ); ?>
                                    </div>
                                <?php endif; ?>
                            </div>
                        <?php endif; ?>
                        <?php endif; ?>
                        <?php if ( count( $product->get_category_ids() ) > 0 ): ?>
                            <span class="posted_in mdp-producter-elementor-product-meta">
                            <span class="mdp-producter-elementor-product-meta-caption">
                               <?php esc_html_e( count( $product->get_category_ids() ) === 1 ? $category_single : $category_plural );  ?>
                            </span>
                            <span class="mdp-producter-elementor-product-meta-content">
                                <?php echo wp_kses_post( get_the_term_list( $product->get_id(), 'product_cat', '', ', ' ) ); ?>
                            </span>
                        </span>
                            <?php if ( $settings['product_meta_separator'] === 'yes' ): ?>
                                <div class="mdp-producter-elementor-product-meta-separator">
                                    <?php if ( $settings['product_meta_separator_type'] === 'text' ) : ?>
                                        <span><?php esc_html_e( $settings['product_meta_separator_text'] ); ?></span>
                                    <?php else: ?>
                                        <div class="mdp-producter-elementor-product-meta-separator-icon">
                                            <?php Icons_Manager::render_icon( $settings['product_meta_separator_icon'], ['aria-hidden' => 'true'] ); ?>
                                        </div>
                                    <?php endif; ?>
                                 </div>
                        <?php endif; ?>
                    <?php endif; ?>
                    <?php if ( count( $product->get_tag_ids() ) > 0 ):  ?>
                        <span class="tagged_as mdp-producter-elementor-product-meta">
                            <span class="mdp-producter-elementor-product-meta-caption">
                                <?php esc_html_e( count( $product->get_category_ids() ) === 1 ? $tag_single : $tag_plural ); ?>
                            </span>
                            <span class="mdp-producter-elementor-product-meta-content">
                                <?php echo wp_kses_post( get_the_term_list( $product->get_id(), 'product_tag', '', ', ' ) ); ?>
                            </span>
                        </span>
                        <?php if ( $settings['product_meta_separator'] === 'yes' ): ?>
                            <div class="mdp-producter-elementor-product-meta-separator">
                                <?php if ( $settings['product_meta_separator_type'] === 'text' ) : ?>
                                    <span><?php esc_html_e( $settings['product_meta_separator_text'] ); ?></span>
                                <?php else: ?>
                                  <div class="mdp-producter-elementor-product-meta-separator-icon">
                                      <?php Icons_Manager::render_icon( $settings['product_meta_separator_icon'], ['aria-hidden' => 'true'] ); ?>
                                  </div>
                                <?php endif; ?>
                            </div>
                        <?php endif; ?>
                    <?php endif; ?>

                    <?php do_action( 'woocommerce_product_meta_end' ); ?>

                </div>

            </div>
        </div>
        <!-- End Producter for Elementor WordPress Plugin -->
	    <?php
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/producter';

    }

}
