<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Worker for Elementor
 * Worker creates a modern and stylish Elementor widget to display business hours.
 * Exclusively on https://1.envato.market/worker-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use DateTime;
use DateTimeZone;
use Exception;
use Elementor\Utils;
use Elementor\Repeater;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Widget_Base;
use Elementor\Icons_Manager;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Text_Shadow;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Worker - Custom Elementor Widget.
 **/
class worker_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
              'mdp-selection-elementor-admin',
                UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-worker',
               UnityPlugin::get_url() . 'css/worker' . UnityPlugin::get_suffix() . '.css',
                   [],
                   UnityPlugin::get_version()
        );

	    wp_register_script(
           'mdp-worker-elementor',
              UnityPlugin::get_url() . 'js/worker-elementor' . UnityPlugin::get_suffix() . '.js',
                  [ 'jquery' ],
                  UnityPlugin::get_version(),
         true
        );

	    /** Get current date and time. */
	    wp_localize_script(
            'mdp-worker-elementor',
            'mdpWorker',
            [
                'timeFormat' => get_option( 'time_format' ),
                'dateFormat' => get_option( 'date_format' ),
                'offset' => self::wc_timezone_offset(),
	        ]
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-worker-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Business Hours', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-worker-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Business Hours', 'business', 'timetable', 'hour', 'schedule' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-worker', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-worker-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /** Enable 24-hour Time format depending on global WP settings. */
        $time_24hr = false;
        $wp_time_format = get_option( 'time_format' );
        if ( ( strpos( $wp_time_format, 'G' ) !== false ) OR ( strpos( $wp_time_format, 'H' ) !== false) ) {
            $time_24hr = true;
        }

        /** Section Header. */
        if ( true ) {
            $this->start_controls_section( 'section_header', [ 'label' => esc_html__( 'Header Content', 'selection' ) ] );

            // What to show in header
            $this->add_control(
                'header_content_type',
                [
                    'label'                 => esc_html__( 'Content Type', 'selection' ),
                    'type'                  => Controls_Manager::CHOOSE,
                    'label_block'       => false,
                    'options'               => [
                        'none' => [
                            'title' => esc_html__( 'Nothing', 'selection' ),
                            'icon' => 'fa fa-ban',
                        ],
                        'date' => [
                            'title' => esc_html__( 'Current Date', 'selection' ),
                            'icon' => 'far fa-calendar-check',
                        ],
                        'status' => [
                            'title' => esc_html__( 'Current Status', 'selection' ),
                            'icon' => 'fa fa-info',
                        ],
                        'icon' => [
                            'title' => esc_html__( 'Icon', 'selection' ),
                            'icon' => 'fa fa-asterisk',
                        ],
                        'image' => [
                            'title' => esc_html__( 'Image', 'selection' ),
                            'icon' => 'far fa-image',
                        ],
                        'text' => [
                            'title' => esc_html__( 'Custom Message', 'selection' ),
                            'icon' => 'fa fa-font',
                        ],
                    ],
                    'default'               => 'date',
                ]
            );

            // Current Date
            $this->add_control(
                'header_date_format',
                [
                    'label'                 => esc_html__( 'Date/Time Format', 'selection' ),
                    'label_block'           => true,
                    'type'                  => Controls_Manager::TEXTAREA,
                    'dynamic'               => ['active'   => true],
                    'default'               => '<h2>{time}</h2>
<span>{date}</span>',
                    'condition'             => [
                        'header_content_type'     => 'date',
                    ],
                ]
            );

            $this->add_control(
                'header_open_msg',
                [
                    'label' => esc_html__( 'Open Message', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic' => ['active' => true ],
                    'default' => esc_html__( 'We are open.', 'selection' ),
                    'placeholder' => esc_html__( 'We are open.', 'selection' ),
                    'condition'             => [
                        'header_content_type'     => 'status',
                    ],
                ]
            );

            $this->add_control(
                'header_closed_msg',
                [
                    'label' => esc_html__( 'Closed Message', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic' => ['active' => true ],
                    'default' => esc_html__( 'Sorry, We are currently closed.', 'selection' ),
                    'placeholder' => esc_html__( 'Sorry, We are closed.', 'selection' ),
                    'condition'             => [
                        'header_content_type'     => 'status',
                    ],
                ]
            );

            // Icon
            $this->add_control(
                'header_icon',
                [
                    'label'					=> esc_html__( 'Icon', 'selection' ),
                    'type'					=> Controls_Manager::ICONS,
                    'fa4compatibility'		=> 'icon',
                    'default'				=> [
                        'value'		=> 'fas fa-asterisk',
                        'library'	=> 'fa-solid',
                    ],
                    'condition'             => [
                        'header_content_type'     => 'icon',
                    ],
                ]
            );

            $this->add_responsive_control(
                'header_icon_size',
                [
                    'label'                 => esc_html__( 'Icon Size', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => 30, 'unit' => 'px'],
                    'range'                 => [
                        'px' => [
                            'min' => 5,
                            'max' => 100,
                            'step' => 1,
                        ]
                    ],
                    'size_units'            => [ 'px', 'em' ],
                    'condition'             => [
                        'header_content_type'     => 'icon',
                    ],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-header .mdp-worker-elementor-icon' => 'font-size: {{SIZE}}{{UNIT}}',
                        '{{WRAPPER}} .mdp-worker-elementor-header .mdp-worker-elementor-icon > svg' => 'height: {{SIZE}}{{UNIT}}'
                    ],
                ]
            );

            // Image
            $this->add_control(
                'header_image',
                [
                    'label'                 => esc_html__( 'Image', 'selection' ),
                    'type'                  => Controls_Manager::MEDIA,
                    'dynamic'               => ['active'   => true],
                    'default'               => [
                        'url' => Utils::get_placeholder_image_src(),
                    ],
                    'condition'             => [
                        'header_content_type' => 'image',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Image_Size::get_type(),
                [
                    'name'      => 'header_image', // Usage: `{name}_size` and `{name}_custom_dimension`, in this case `image_size` and `image_custom_dimension`.
                    'default'   => 'full',
                    'separator' => 'none',
                    'condition'             => [
                        'header_content_type' => 'image',
                    ],
                ]
            );

            $this->add_responsive_control(
                'header_img_width',
                [
                    'label'                 => esc_html__( 'Width', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => '', 'unit'  => 'px'],
                    'range'                 => [
                        'px' => [
                            'min'   => 10,
                            'max'   => 500,
                            'step'  => 10,
                        ],
                        '%' => [
                            'min'   => 3,
                            'max'   => 100,
                            'step'  => 1,
                        ],
                    ],
                    'size_units'            => ['px', '%'],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-header .mdp-worker-elementor-image img' => 'width: {{SIZE}}{{UNIT}}',
                    ],
                    'condition'             => [
                        'header_content_type'     => 'image',
                    ],
                ]
            );

            // Custom Message
            $this->add_control(
                'header_text',
                [
                    'label'                 => esc_html__( 'Custom Message', 'selection' ),
                    'label_block'           => true,
                    'type'                  => Controls_Manager::TEXTAREA,
                    'dynamic'               => ['active'   => true],
                    'default'               => esc_html__( 'Your Custom Message', 'selection' ),
                    'condition'             => [
                        'header_content_type'     => 'text',
                    ],
                ]
            );

            $this->add_responsive_control(
                'header_content_position',
                [
                    'label' => __( 'Position', 'plugin-domain' ),
                    'type' => Controls_Manager::SELECT,
                    'options' => [
                        'mdp-top-left'      => esc_html__( 'Top Left', 'selection' ),
                        'mdp-top-center'    => esc_html__( 'Top Center', 'selection' ),
                        'mdp-top-right'     => esc_html__( 'Top Right', 'selection' ),
                        'mdp-center-left'   => esc_html__( 'Center Left', 'selection' ),
                        'mdp-center-center' => esc_html__( 'Center Center', 'selection' ),
                        'mdp-center-right'  => esc_html__( 'Center Right', 'selection' ),
                        'mdp-bottom-left'   => esc_html__( 'Bottom Left', 'selection' ),
                        'mdp-bottom-center' => esc_html__( 'Bottom Center', 'selection' ),
                        'mdp-bottom-right'  => esc_html__( 'Bottom Right', 'selection' ),
                    ],
                    'default' => 'mdp-center-left',
                    'condition'             => [
                        'header_content_type!'     => 'none',
                    ],
                ]
            );

            $this->end_controls_section();
        }

        /** Section Business Hours. */
        if ( true ) {
            $this->start_controls_section( 'section_business_hours', ['label' => esc_html__( 'Business Hours', 'selection' )] );

            $business_hours_repeater = new Repeater();

            $business_hours_repeater->add_control(
                'start_time',
                [
                    'label' => esc_html__( 'Start Time', 'selection' ),
                    'label_block' => false,
                    'type' => Controls_Manager::DATE_TIME,
                    'default' =>  $this->format_time( '9:00' ),//'H:i'
                    'picker_options' => ['enableTime' => true, 'noCalendar' => true, 'dateFormat' => $this->time_format_to_js( $wp_time_format ), 'time_24hr' => $time_24hr]
                ]
            );

            $business_hours_repeater->add_control(
                'end_time',
                [
                    'label' => esc_html__( 'End Time', 'selection' ),
                    'label_block' => false,
                    'type' => Controls_Manager::DATE_TIME,
                    'default' =>  $this->format_time( '20:00' ),//'H:i'
                    'picker_options' => ['enableTime' => true, 'noCalendar' => true, 'dateFormat' => $this->time_format_to_js( $wp_time_format ), 'time_24hr' => $time_24hr]
                ]
            );

            /** Days of week. */
            $week = [
                'sun' => esc_html__( 'Sunday', 'selection' ),
                'mon' => esc_html__( 'Monday', 'selection' ),
                'tue' => esc_html__( 'Tuesday', 'selection' ),
                'wed' => esc_html__( 'Wednesday', 'selection' ),
                'thu' => esc_html__( 'Thursday', 'selection' ),
                'fri' => esc_html__( 'Friday', 'selection' ),
                'sat' => esc_html__( 'Saturday', 'selection' ),
            ];

            /** Add array offset, to set correct first day of week. */
            $week = $this->set_start_of_week( $week );

            /** Create control foreach day of week. */
            $count = 0;
            foreach ( $week as $key => $day ) {
                $count++;

                /** Header. */
                $html = "<h4><strong>{$day}</strong></h4>";
                $this->add_control(
                    "{$key}_header",
                    [
                        'type' => Controls_Manager::RAW_HTML,
                        'raw' => $html,
                    ]
                );

                /** Day Label. */
                $this->add_control(
                    "{$key}_day_label",
                    [
                        'label' => esc_html__( 'Day Label:', 'selection' ),
                        'label_block' => false,
                        'type' => Controls_Manager::TEXT,
                        'default' => $day,
                    ]
                );

                /** Closed. */
                $default = '';
                if ( $count > 5 ) { $default = 'yes'; }
                $this->add_control(
                    "{$key}_closed",
                    [
                        'label'             => esc_html__( 'Closed All Day:', 'selection' ),
                        'type'              => Controls_Manager::SWITCHER,
                        'default'           => $default,
                        'label_on'          => esc_html__( 'Yes', 'selection' ),
                        'label_off'         => esc_html__( 'No', 'selection' ),
                        'return_value'      => 'yes',
                    ]
                );

                $this->add_control(
                    "{$key}_closed_all_day_msg",
                    [
                        'label'         => esc_html__( 'Closed All Day Message:', 'selection' ),
                        'label_block'   => true,
                        'type'          => Controls_Manager::TEXT,
                        'dynamic'       => ['active' => true ],
                        'default'       => esc_html__( 'Closed All Day', 'selection' ),
                        'placeholder'   => esc_html__( 'Closed All Day', 'selection' ),
                        'condition'     => [
                            "{$key}_closed" => 'yes'
                        ]
                    ]
                );

                /** Open 24 hours. */
                $this->add_control(
                    "{$key}_open_24",
                    [
                        'label'             => esc_html__( 'Open 24 Hours:', 'selection' ),
                        'type'              => Controls_Manager::SWITCHER,
                        'default'           => '',
                        'label_on'          => esc_html__( 'Yes', 'selection' ),
                        'label_off'         => esc_html__( 'No', 'selection' ),
                        'return_value'      => 'yes',
                        'condition'         => [
                            "{$key}_closed" => ''
                        ]
                    ]
                );

                $this->add_control(
                    "{$key}_open_all_day_msg",
                    [
                        'label'         => esc_html__( 'Open 24 hours message:', 'selection' ),
                        'label_block'   => true,
                        'type'          => Controls_Manager::TEXT,
                        'dynamic'       => ['active' => true ],
                        'default'       => esc_html__( 'Open 24 hours', 'selection' ),
                        'placeholder'   => esc_html__( 'Open 24 hours', 'selection' ),
                        'condition'     => [
                            "{$key}_open_24" => 'yes',
                            "{$key}_closed" => ''
                        ]
                    ]
                );

                /** Business Hours. */
                $this->add_control(
                    "{$key}_business_hours",
                    [
                        'label' => esc_html__( 'Business Hours:', 'selection' ),
                        'type' => Controls_Manager::REPEATER,
                        'fields' => $business_hours_repeater->get_controls(),
                        'prevent_empty' => false,
                        'default' => [
                            [
                                'start_time' => $this->format_time( '9:00' ),//'H:i',
                                'end_time' => $this->format_time( '20:00' ),//'H:i'
                            ]
                        ],
                        'title_field' => '{{{ start_time }}} - {{{ end_time }}}',
                        'condition'         => [
                            "{$key}_closed" => '',
                            "{$key}_open_24" => '',
                        ],
                    ]
                );

                /** Separator. */
                $this->add_control( "{$key}_separator", ['type' => Controls_Manager::DIVIDER, 'style' => 'thick'] );

            } // End Foreach.

            $this->end_controls_section();

        }

        /** Holidays & Special Days. */
        if ( true ) {

            $this->start_controls_section( 'section_holidays', [ 'label' => esc_html__( 'Holidays & Special Days', 'selection' ) ] );

            $this->add_control(
                'holidays_dates',
                [
                    'label'         => esc_html__( 'Holidays:', 'selection' ),
                    'label_block'   => true,
                    'type'          => Controls_Manager::DATE_TIME,
                    'picker_options'    => [
                        'enableTime'    => false,
                        'dateFormat'    => 'd-m-Y',
                        'mode'          => 'multiple'
                    ]
                ]
            );

            $this->add_control(
                "holidays_recurring",
                [
                    'label'             => esc_html__( 'Recurring:', 'selection' ),
                    'type'              => Controls_Manager::SWITCHER,
                    'description'       => esc_html__( 'Enable to apply date range every year. So the year in the selection will be ignored.', 'selection' ),
                    'default'           => 'yes',
                    'label_on'          => esc_html__( 'Yes', 'selection' ),
                    'label_off'         => esc_html__( 'No', 'selection' ),
                    'return_value'      => 'yes',
                ]
            );

            $this->add_control(
                'holidays_msg',
                [
                    'label'         => esc_html__( 'Holiday Message', 'selection' ),
                    'label_block'   => true,
                    'type'          => Controls_Manager::TEXT,
                    'dynamic'       => ['active' => true ],
                    'default'       => esc_html__( 'Special holiday hours in effect.', 'selection' ),
                    'placeholder'   => esc_html__( 'Enter message for Holidays.', 'selection' ),
                ]
            );

            $this->end_controls_section();
        }

        /** Section Footer. */
        if ( true ) {
            $this->start_controls_section( 'section_footer', [ 'label' => esc_html__( 'Footer Content', 'selection' ) ] );

            // What to show in footer
            $this->add_control(
                'footer_content_type',
                [
                    'label'                 => esc_html__( 'Content Type', 'selection' ),
                    'type'                  => Controls_Manager::CHOOSE,
                    'label_block'       => false,
                    "options"               => [
                        'none' => [
                            'title' => esc_html__( 'Nothing', 'selection' ),
                            'icon' => 'fa fa-ban',
                        ],
                        "date" => [
                            'title' => esc_html__( 'Current Date', 'selection' ),
                            'icon' => 'far fa-calendar-check',
                        ],
                        'status' => [
                            'title' => esc_html__( 'Current Status', 'selection' ),
                            'icon' => 'fa fa-info',
                        ],
                        'icon' => [
                            'title' => esc_html__( 'Icon', 'selection' ),
                            'icon' => 'fa fa-asterisk',
                        ],
                        'image' => [
                            'title' => esc_html__( 'Image', 'selection' ),
                            'icon' => 'far fa-image',
                        ],
                        'text' => [
                            'title' => esc_html__( 'Custom Message', 'selection' ),
                            'icon' => 'fa fa-font',
                        ],
                    ],
                    'default' => 'status',
                ]
            );

            // Current Date
            $this->add_control(
                'footer_date_format',
                [
                    'label'                 => esc_html__( 'Date/Time Format', 'selection' ),
                    'label_block'           => true,
                    'type'                  => Controls_Manager::TEXTAREA,
                    'dynamic'               => ['active'   => true],
                    'default'               => '<h2>{time}</h2>
<span>{date}</span>',
                    'condition'             => [
                        'footer_content_type'     => 'date',
                    ],
                ]
            );

            $this->add_control(
                'footer_open_msg',
                [
                    'label' => esc_html__( 'Open Message', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic' => ['active' => true ],
                    'default' => esc_html__( 'We are open.', 'selection' ),
                    'placeholder' => esc_html__( 'We are open.', 'selection' ),
                    'condition'             => [
                        'footer_content_type'     => 'status',
                    ],
                ]
            );

            $this->add_control(
                'footer_closed_msg',
                [
                    'label' => esc_html__( 'Closed Message', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic' => ['active' => true ],
                    'default' => esc_html__( 'Sorry, We are currently closed.', 'selection' ),
                    'placeholder' => esc_html__( 'Sorry, We are closed.', 'selection' ),
                    'condition'             => [
                        'footer_content_type'     => 'status',
                    ],
                ]
            );

            // Icon
            $this->add_control(
                'footer_icon',
                [
                    'label'					=> esc_html__( 'Icon', 'selection' ),
                    'type'					=> Controls_Manager::ICONS,
                    'fa4compatibility'		=> 'icon',
                    'default'				=> [
                        'value'		=> 'fas fa-asterisk',
                        'library'	=> 'fa-solid',
                    ],
                    'condition'             => [
                        'footer_content_type'     => 'icon',
                    ],
                ]
            );

            $this->add_responsive_control(
                'footer_icon_size',
                [
                    'label'                 => esc_html__( 'Icon Size', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => 30, 'unit' => 'px'],
                    'range'                 => [
                        'px' => [
                            'min' => 5,
                            'max' => 100,
                            'step' => 1,
                        ]
                    ],
                    'size_units'            => [ 'px', 'em' ],
                    'condition'             => [
                        'footer_content_type'     => 'icon',
                    ],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-footer .mdp-worker-elementor-icon' => 'font-size: {{SIZE}}{{UNIT}}',
                        '{{WRAPPER}} .mdp-worker-elementor-footer .mdp-worker-elementor-icon > svg' => 'height: {{SIZE}}{{UNIT}}'
                    ],
                ]
            );

            // Image
            $this->add_control(
                'footer_image',
                [
                    'label'                 => esc_html__( 'Image', 'selection' ),
                    'type'                  => Controls_Manager::MEDIA,
                    'dynamic'               => ['active'   => true],
                    'default'               => [
                        'url' => Utils::get_placeholder_image_src(),
                    ],
                    'condition'             => [
                        'footer_content_type' => 'image',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Image_Size::get_type(),
                [
                    'name'      => 'footer_image', // Usage: `{name}_size` and `{name}_custom_dimension`, in this case `image_size` and `image_custom_dimension`.
                    'default'   => 'full',
                    'separator' => 'none',
                    'condition'             => [
                        'footer_content_type' => 'image',
                    ],
                ]
            );

            $this->add_responsive_control(
                'footer_img_width',
                [
                    'label'                 => esc_html__( 'Width', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => '', 'unit'  => 'px'],
                    'range'                 => [
                        'px' => [
                            'min'   => 10,
                            'max'   => 500,
                            'step'  => 10,
                        ],
                        '%' => [
                            'min'   => 3,
                            'max'   => 100,
                            'step'  => 1,
                        ],
                    ],
                    'size_units'            => ['px', '%'],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-footer .mdp-worker-elementor-image img' => 'width: {{SIZE}}{{UNIT}}',
                    ],
                    'condition'             => [
                        'footer_content_type'     => 'image',
                    ],
                ]
            );

            // Custom Message
            $this->add_control(
                'footer_text',
                [
                    'label'                 => esc_html__( 'Custom Message', 'selection' ),
                    'label_block'           => true,
                    'type'                  => Controls_Manager::TEXTAREA,
                    'dynamic'               => ['active'   => true],
                    'default'               => esc_html__( 'Your Custom Message', 'selection' ),
                    'condition'             => [
                        'footer_content_type'     => 'text',
                    ],
                ]
            );

            $this->add_responsive_control(
                'footer_content_position',
                [
                    'label' => __( 'Position', 'plugin-domain' ),
                    'type' => Controls_Manager::SELECT,
                    'options' => [
                        'mdp-top-left'      => esc_html__( 'Top Left', 'selection' ),
                        'mdp-top-center'    => esc_html__( 'Top Center', 'selection' ),
                        'mdp-top-right'     => esc_html__( 'Top Right', 'selection' ),
                        'mdp-center-left'   => esc_html__( 'Center Left', 'selection' ),
                        'mdp-center-center' => esc_html__( 'Center Center', 'selection' ),
                        'mdp-center-right'  => esc_html__( 'Center Right', 'selection' ),
                        'mdp-bottom-left'   => esc_html__( 'Bottom Left', 'selection' ),
                        'mdp-bottom-center' => esc_html__( 'Bottom Center', 'selection' ),
                        'mdp-bottom-right'  => esc_html__( 'Bottom Right', 'selection' ),
                    ],
                    'default' => 'mdp-center-left',
                    'condition'             => [
                        'footer_content_type!'     => 'none',
                    ],
                ]
            );

            $this->end_controls_section();
        }

        /** Header Style tab. */
        if ( true ) {
            $this->start_controls_section( 'header_style_section', ['label' => esc_html__( 'Header', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE]);

            // Hide
            $this->add_control(
                "show_header",
                [
                    'label'             => esc_html__( 'Show Header', 'selection' ),
                    'type'              => Controls_Manager::SWITCHER,
                    'default'           => 'yes',
                    'label_on'          => esc_html__( 'Yes', 'selection' ),
                    'label_off'         => esc_html__( 'No', 'selection' ),
                    'return_value'      => 'yes',
                ]
            );

            $this->add_responsive_control(
                'header_height',
                [
                    'label'                 => esc_html__( 'Height', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => 120, 'unit' => 'px'],
                    'range'                 => [
                        'px' => [
                            'min' => 5,
                            'max' => 500,
                            'step' => 1,
                        ]
                    ],
                    'size_units'            => [ 'px' ],
                    'condition'         => [
                        'show_header' => 'yes'
                    ],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-header' => 'height: {{SIZE}}{{UNIT}}',
                    ],
                ]
            );

            $this->add_responsive_control(
                'header_padding',
                [
                    'label'         => esc_html__( 'Padding', 'selection' ),
                    'type'          => Controls_Manager::DIMENSIONS,
                    'default'       => [ 'top' => 0, 'right' => 20, 'bottom' => 0, 'left' => 20 ],
                    'size_units'    => [ 'px', 'em', '%' ],
                    'selectors'     => [
                        '{{WRAPPER}} .mdp-worker-elementor-header' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            $this->add_control(
                'header_radius',
                [
                    'label' => __( 'Border Radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'default'       => [ 'top' => 4, 'right' => 4, 'bottom' => 4, 'left' => 4 ],
                    'size_units' => [ 'px', '%' ],
                    'selectors' => [
                        '{{WRAPPER}} .mdp-worker-elementor-header' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_header' => 'yes'
                    ],
                    'separator' => 'after',
                ]
            );

            // Background
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name'              => 'header_background',
                    'types'             => [ 'classic', 'gradient' ],
                    'selector'          => '{{WRAPPER}} .mdp-worker-elementor-header',
                    'condition'         => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            // Text Color
            $this->add_control(
                'header_color',
                [
                    'label'     => esc_html__( 'Text Color', 'selection' ),
                    'separator' => 'before',
                    'type'      => Controls_Manager::COLOR,
                    'scheme'    => [
                        'type'  => Color::get_type(),
                        'value' => Color::COLOR_1,
                    ],
                    'selectors' => [
                        // Stronger selector to avoid section style from overwriting
                        '{{WRAPPER}} .mdp-worker-elementor-widget-box > .mdp-worker-elementor-header > .mdp-worker-elementor-header-content' => 'color: {{VALUE}};',
                    ],
                    'condition' => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            // Typography
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name'      => 'header_typography',
                    'scheme'    => Typography::TYPOGRAPHY_1,
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-header-content',
                    'condition' => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            //Text Shadow
            $this->add_group_control(
                Group_Control_Text_Shadow::get_type(),
                [
                    'name'      => 'header_text_shadow',
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-header-content',
                    'condition' => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            $this->end_controls_section();
        }

        /** Business Hours Style tab. */
        if ( true ) {
            $this->start_controls_section( 'bh_style_section', ['label' => esc_html__( 'Business Hours', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE]);

            $this->add_control(
                'show_bh',
                [
                    'label'             => esc_html__( 'Show Business Hours', 'selection' ),
                    'type'              => Controls_Manager::SWITCHER,
                    'default'           => 'yes',
                    'label_on'          => esc_html__( 'Yes', 'selection' ),
                    'label_off'         => esc_html__( 'No', 'selection' ),
                    'return_value'      => 'yes',
                ]
            );

            $this->add_responsive_control(
                'bh_margin',
                [
                    'label'         => esc_html__( 'Margin', 'selection' ),
                    'type'          => Controls_Manager::DIMENSIONS,
                    'size_units'    => [ 'px', 'em', '%' ],
                    'selectors'     => [
                        '{{WRAPPER}} .mdp-worker-elementor-hours' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_bh' => 'yes'
                    ]
                ]
            );

            $this->add_responsive_control(
                'bh_padding',
                [
                    'label'         => esc_html__( 'Padding', 'selection' ),
                    'type'          => Controls_Manager::DIMENSIONS,
                    'default'       => [ 'top' => 20, 'right' => 20, 'bottom' => 20, 'left' => 20 ],
                    'size_units'    => [ 'px', 'em', '%' ],
                    'selectors'     => [
                        '{{WRAPPER}} .mdp-worker-elementor-hours' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_bh' => 'yes'
                    ]
                ]
            );

            $this->add_control(
                'bh_radius',
                [
                    'label' => __( 'Border Radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'default'       => [ 'top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0 ],
                    'size_units' => [ 'px', '%' ],
                    'selectors' => [
                        '{{WRAPPER}} .mdp-worker-elementor-hours' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_bh' => 'yes'
                    ],
                    'separator' => 'after'
                ]
            );

            // Background
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name'              => 'bh_background',
                    'types'             => [ 'classic', 'gradient' ],
                    'selector'          => '{{WRAPPER}} .mdp-worker-elementor-hours',
                    'condition'         => [
                        'show_bh' => 'yes'
                    ]
                ]
            );

            // Text Color
            $this->add_control(
                'bh_color',
                [
                    'label'     => esc_html__( 'Text Color', 'selection' ),
                    'separator' => 'before',
                    'type'      => Controls_Manager::COLOR,
                    'scheme'    => [
                        'type'  => Color::get_type(),
                        'value' => Color::COLOR_2,
                    ],
                    'selectors' => [
                        // Stronger selector to avoid section style from overwriting
                        '{{WRAPPER}} .mdp-worker-elementor-widget-box > .mdp-worker-elementor-hours > .mdp-worker-elementor-row' => 'color: {{VALUE}};',
                    ],
                    'condition' => [
                        'show_bh' => 'yes'
                    ]
                ]
            );

            // Typography
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name'      => 'bh_typography',
                    'scheme'    => Typography::TYPOGRAPHY_1,
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-hours',
                    'condition' => [
                        'show_bh' => 'yes'
                    ]
                ]
            );

            //Text Shadow
            $this->add_group_control(
                Group_Control_Text_Shadow::get_type(),
                [
                    'name'      => 'bh_text_shadow',
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-hours',
                    'condition' => [
                        'show_header' => 'yes'
                    ]
                ]
            );

            $this->add_responsive_control(
                'row_margin',
                [
                    'label'             => __( 'Space Between', 'selection' ),
                    'type'              => Controls_Manager::SLIDER,
                    'default'           => [ 'size' => 4 ],
                    'range'             => [
                        'px' => [
                            'min'   => 0,
                            'max'   => 100,
                            'step'  => 1,
                        ],
                    ],
                    'size_units'        => [ 'px' ],
                    'selectors'         => [
                        '{{WRAPPER}} .mdp-worker-elementor-hours .mdp-worker-elementor-row:not(:last-child)' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                    ],
                ]
            );

            $this->end_controls_section();
        }

        /** Footer Style tab. */
        if ( true ) {
            $this->start_controls_section( 'footer_style_section', ['label' => esc_html__( 'Footer', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE]);

            // Hide
            $this->add_control(
                "show_footer",
                [
                    'label'             => esc_html__( 'Show Footer', 'selection' ),
                    'type'              => Controls_Manager::SWITCHER,
                    'default'           => 'yes',
                    'label_on'          => esc_html__( 'Yes', 'selection' ),
                    'label_off'         => esc_html__( 'No', 'selection' ),
                    'return_value'      => 'yes',
                ]
            );

            $this->add_responsive_control(
                'footer_height',
                [
                    'label'                 => esc_html__( 'Height', 'selection' ),
                    'type'                  => Controls_Manager::SLIDER,
                    'default'               => ['size' => 40, 'unit' => 'px'],
                    'range'                 => [
                        'px' => [
                            'min' => 5,
                            'max' => 500,
                            'step' => 1,
                        ]
                    ],
                    'size_units'            => [ 'px' ],
                    'condition'         => [
                        'show_footer' => 'yes'
                    ],
                    'selectors'             => [
                        '{{WRAPPER}} .mdp-worker-elementor-footer' => 'height: {{SIZE}}{{UNIT}}',
                    ],
                ]
            );

            $this->add_responsive_control(
                'footer_padding',
                [
                    'label'         => esc_html__( 'Padding', 'selection' ),
                    'type'          => Controls_Manager::DIMENSIONS,
                    'default'       => [ 'top' => 0, 'right' => 20, 'bottom' => 0, 'left' => 20 ],
                    'size_units'    => [ 'px', 'em', '%' ],
                    'selectors'     => [
                        '{{WRAPPER}} .mdp-worker-elementor-footer' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_footer' => 'yes'
                    ],
                    'separator' => 'after',
                ]
            );

            $this->add_control(
                'footer_radius',
                [
                    'label' => __( 'Border Radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => [ 'px', '%' ],
                    'selectors' => [
                        '{{WRAPPER}} .mdp-worker-elementor-footer' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                    'condition'         => [
                        'show_footer' => 'yes'
                    ]
                ]
            );

            // Background
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name'              => 'footer_background',
                    'types'             => [ 'classic', 'gradient' ],
                    'selector'          => '{{WRAPPER}} .mdp-worker-elementor-footer',
                    'condition'         => [
                        'show_footer' => 'yes'
                    ]
                ]
            );

            // Text Color
            $this->add_control(
                'footer_color',
                [
                    'label'     => esc_html__( 'Text Color', 'selection' ),
                    'separator' => 'before',
                    'type'      => Controls_Manager::COLOR,
                    'scheme'    => [
                        'type'  => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        // Stronger selector to avoid section style from overwriting
                        '{{WRAPPER}} .mdp-worker-elementor-widget-box > .mdp-worker-elementor-footer > .mdp-worker-elementor-footer-content' => 'color: {{VALUE}};',
                    ],
                    'condition' => [
                        'show_footer' => 'yes'
                    ]
                ]
            );

            // Typography
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name'      => 'footer_typography',
                    'scheme'    => Typography::TYPOGRAPHY_1,
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-footer-content',
                    'condition' => [
                        'show_footer' => 'yes'
                    ]
                ]
            );

            //Text Shadow
            $this->add_group_control(
                Group_Control_Text_Shadow::get_type(),
                [
                    'name'      => 'footer_text_shadow',
                    'selector'  => '{{WRAPPER}} .mdp-worker-elementor-footer-content',
                    'condition' => [
                        'show_footer' => 'yes'
                    ]
                ]
            );

            $this->end_controls_section();
        }

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** Get widget settings. */
        $s = $this->get_settings_for_display();

        ?>
        <!-- Start Worker for Elementor WordPress Plugin -->
        <div class="mdp-worker-elementor-widget-box">

            <?php if ( $s['show_header'] ) : ?>
                <div class="mdp-worker-elementor-header <?php esc_attr_e( $s['header_content_position'] ); ?>">

                    <div class="mdp-worker-elementor-header-content">
                        <?php if ( 'date' === $s['header_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-date">
                                <?php
                                /** Localize current date */
                                $current_time = date_i18n( get_option( 'time_format' ) );
                                $current_date = date_i18n( get_option( 'date_format' ) );

                                /** Make replacements. */
                                $date_result = str_replace( '{time}', $current_time, $s['header_date_format'] );
                                $date_result = str_replace( '{date}', $current_date, $date_result );

                                /** Print result. */
                                echo wp_kses_post( $date_result );
                                ?>
                            </div>
                        <?php elseif ( 'status' === $s['header_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-status">
                                <?php if ( $this->is_open( $s ) ) : ?>
                                    <?php echo wp_kses_post( $s['header_open_msg'] ); ?>
                                <?php else : ?>
                                    <?php echo wp_kses_post( $s['header_closed_msg'] ); ?>
                                <?php endif; ?>
                            </div>
                        <?php elseif ( 'icon' === $s['header_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-icon">
                                <?php Icons_Manager::render_icon( $s['header_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            </div>
                        <?php elseif ( 'image' === $s['header_content_type'] ) : ?>
                            <?php if ( ! empty( $s['header_image']['url'] ) ) : ?>
                                <div class="mdp-worker-elementor-image">
                                    <?php echo Group_Control_Image_Size::get_attachment_image_html( $s, 'header_image', 'header_image' ); ?>
                                </div>
                            <?php endif; ?>
                        <?php elseif ( 'text' === $s['header_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-text">
                                <?php echo do_shortcode( $s['header_text'] ); ?>
                            </div>
                        <?php endif; ?>
                    </div>

                </div>
            <?php endif; ?>

            <?php if ( $s['show_bh'] ) : ?>

                <div class="mdp-worker-elementor-hours">
                    <?php $week = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat']; /** Days of week. */ ?>
                    <?php $week = $this->set_start_of_week( $week ); ?>

                    <?php foreach ( $week as $day ) : ?>
                        <div class="mdp-worker-elementor-row">
                            <div class="mdp-worker-elementor-row-day">
                                <?php echo wp_kses_post( $s["{$day}_day_label"] ); ?>
                            </div>
                            <div class="mdp-worker-elementor-row-hours">
                                <?php if ( $this->is_holiday_today( $s['holidays_dates'], $s['holidays_recurring'], $day ) ) : ?>
                                    <?php echo wp_kses_post( $s["holidays_msg"] ); ?>
                                <?php elseif ( 'yes' === $s["{$day}_closed"] ) : ?>
                                    <?php echo wp_kses_post( $s["{$day}_closed_all_day_msg"] ); ?>
                                <?php elseif ( 'yes' === $s["{$day}_open_24"] ) : ?>
                                    <?php echo wp_kses_post( $s["{$day}_open_all_day_msg"] ); ?>
                                <?php else : ?>
                                    <?php foreach ( $s["{$day}_business_hours"] as $hours ) : ?>
                                        <?php esc_html_e( $this->format_time_to_wp( $hours['start_time'] ) ); ?> - <?php esc_html_e( $this->format_time_to_wp( $hours['end_time'] ) ); ?>
                                    <?php endforeach; ?>
                                <?php endif; ?>
                            </div>
                        </div>
                    <?php endforeach; ?>

                </div>
            <?php endif; ?>

            <?php if ( $s['show_footer'] ) : ?>
                <div class="mdp-worker-elementor-footer <?php esc_attr_e( $s['footer_content_position'] ); ?>">

                    <div class="mdp-worker-elementor-footer-content">
                        <?php if ( 'date' === $s['footer_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-date">
                                <?php
                                /** Get current date and time. */
                                $current_time = current_time( get_option( 'time_format' ) );
                                $current_date = current_time( get_option( 'date_format' ) );

                                /** Make replacements. */
                                $date_result = str_replace( '{time}', $current_time, $s['footer_date_format'] );
                                $date_result = str_replace( '{date}', $current_date, $date_result );

                                /** Print result. */
                                echo wp_kses_post( $date_result );
                                ?>
                            </div>
                        <?php elseif ( 'status' === $s['footer_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-status">
                                <?php if ( $this->is_open( $s ) ) : ?>
                                    <?php echo wp_kses_post( $s['footer_open_msg'] ); ?>
                                <?php else : ?>
                                    <?php echo wp_kses_post( $s['footer_closed_msg'] ); ?>
                                <?php endif; ?>
                            </div>
                        <?php elseif ( 'icon' === $s['footer_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-icon">
                                <?php Icons_Manager::render_icon( $s['footer_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            </div>
                        <?php elseif ( 'image' === $s['footer_content_type'] ) : ?>
                            <?php if ( ! empty( $s['footer_image']['url'] ) ) : ?>
                                <div class="mdp-worker-elementor-image">
                                    <?php echo Group_Control_Image_Size::get_attachment_image_html( $s, 'footer_image', 'footer_image' ); ?>
                                </div>
                            <?php endif; ?>
                        <?php elseif ( 'text' === $s['footer_content_type'] ) : ?>
                            <div class="mdp-worker-elementor-text">
                                <?php echo do_shortcode( $s['footer_text'] ); ?>
                            </div>
                        <?php endif; ?>
                    </div>

                </div>
            <?php endif; ?>

        </div>
        <!-- End Worker for Elementor WordPress Plugin -->
        <?php
    }

    /**
     * Return time in WP format.
     * @param $time
     *
     * @return false|string
     */
    private function format_time_to_wp( $time ) {

        /** WordPress time format. */
        $wp_time_format = get_option( 'time_format' );

        /** Use $time if time format have escaped symbols */
	    $pm = preg_match('/(\\\)/', $wp_time_format );
        return $pm > 0 ? $time : date( $wp_time_format, strtotime( $time ) );

    }

    /**
     * Add array offset, to set correct first day of week.
     *
     *
     * @param $week array - Days of week, starting from Sun.
     *
     * @since  1.0.0
     * @access public
     * @return array - Days of week with correct first day.
     */
    private function set_start_of_week( $week ) {

        /** Get WP Start day of the week. */
        $start_of_week = get_option( 'start_of_week' );

        /** Add offset to array. */
        for( $i = 0; $i < $start_of_week; $i++ ) {
            $this->array_shift ( $week );
        }

        return $week;
    }

    /**
     * Return time in WP format.
     *
     *
     * @param $time string - Time in format 'H:i'
     *
     * @since  1.0.0
     * @access public
     * @return string - Time in current WP format.
     **/
    private function format_time( $time ) {

        /** Get WP time format. */
        $wp_time_format = get_option( 'time_format' );

        $time_obj = DateTime::createFromFormat( 'H:i', $time );

        if ( ! $time_obj ) {  return $time; }

        return $time_obj->format( $wp_time_format );

    }

    /**
     * Make array offset.
     *
     * @param $arr array - Reference to array on which we will carry out manipulations.
     *
     * @since  1.0.0
     * @access public
     * @return void
     **/
    private function array_shift ( &$arr ) {

        $keys = array_keys( $arr );
        $val = $arr[$keys[0]];
        unset( $arr[$keys[0]] );
        $arr[$keys[0]] = $val;

    }

    /**
     * Convert php time format string to js.
     * @see https://flatpickr.js.org/formatting/
     *
     * @param $time_format string - PHP Time in format, ex. 'g:i A'
     *
     * @since  1.0.0
     * @access public
     * @return string - Time in current WP format.
     **/
    private function time_format_to_js( $time_format ) {

        $js_format = $time_format;

        // AM/PM
        $js_format = str_replace( 'a', 'K', $js_format );
        $js_format = str_replace( 'A', 'K', $js_format );

        // g => G
        $js_format = str_replace( 'g', 'G', $js_format );

        return $js_format;
    }

    /**
     * Return current status.
     *
     * @param $settings - All widget settings.
     *
     * @return bool - True if we are open now.
     * @since 1.0.0
     * @access public
     **/
    private function is_open( $settings ) {

        /** Get current day prefix. */
        $day = strtolower( current_time( 'D' ) ); // mon

        /** Check if holiday today. */
        if ( $this->is_holiday( $settings['holidays_dates'], $settings['holidays_recurring'] ) ) {
            return false;
        }

        /** If we are closed all day. */
        if ( 'yes' === $settings["{$day}_closed"] ) {
            return false;
        }

        /** If we are open all day. */
        if ( 'yes' === $settings["{$day}_open_24"] ) {
            return true;
        }

        /** Check, are we opened now? */
        foreach ( $settings["{$day}_business_hours"] as $hours ) {

            /** Prepare variables. */
            $wp_time_format = get_option( 'time_format' );
            $current_time = current_time( $wp_time_format );

            $start_time = $hours['start_time'];
            $end_time = $hours['end_time'];

            /** Convert to same format. */
            $date1 = DateTime::createFromFormat( $wp_time_format, $current_time );
            $date2 = DateTime::createFromFormat( $wp_time_format, $start_time );
            $date3 = DateTime::createFromFormat( $wp_time_format, $end_time );

            /** If the current time between start_time and end_time - we are opened now. */
            if ( $date1 > $date2 && $date1 < $date3 ) {
                return true;
            }
        }

        /** We are closed by default. */
        return false;
    }

    /**
     * Return true, if today is holiday.
     *
     * @param $dates string - Comma separated holiday dates.
     *
     * @param $recurring string - 'yes' if we need to apply date range every year. So the year in the dates will be ignored.
     *
     * @return bool - true if today is holiday.
     * @since 1.0.0
     * @access public
     */
    private function is_holiday( $dates, $recurring ) {

        /** Convert $dates to array. */
        $dates = explode( ', ', $dates );

        /** Convert $recurring to bool. */
        if ( 'yes' === $recurring ) {
            $recurring = true;
        } else {
            $recurring = false;
        }

        /** Get today date. */
        $today = date( 'd-m-Y' );
        if ( $recurring ) {
            $today = substr( $today, 0, -5 );
        }

        /** Search holiday. */
        foreach ( $dates as $date ) {

            if ( $recurring ) {
                $date = substr( $date, 0, -5 );
            }

            /** Today is holiday. */
            if ( $today === $date ) {
                return true;
            }

        }

        /** Today is typical day. */
        return false;
    }

    /**
     * Return true, if today is holiday for week day.
     *
     * @param $dates string - Comma separated holiday dates.
     * @param $recurring string - 'yes' if we need to apply date range every year. So the year in the dates will be ignored.
     * @param $day string - Current week day in short, ex.: mon
     *
     * @return bool - true if today is holiday.
     * @since 1.0.0
     * @access public
     */
    private function is_holiday_today( $dates, $recurring, $day ) {

        /** Convert $dates to array. */
        $dates = explode( ', ', $dates );

        /** Convert $recurring to bool. */
        if ( 'yes' === $recurring ) {
            $recurring = true;
        } else {
            $recurring = false;
        }

        /** Get today date. */
        $today = date( 'd-m-Y', strtotime( "{$day} this week" ) );
        if ( $recurring ) {
            $today = substr( $today, 0, -5 );
        }

        /** Search holiday. */
        foreach ( $dates as $date ) {

            if ( $recurring ) {
                $date = substr( $date, 0, -5 );
            }

            /** Today is holiday. */
            if ( $today === $date ) {
                return true;
            }

        }

        /** Today is typical day. */
        return false;
    }

	/**
     * Get timezone offset
	 * @return float|int
	 */
	private function wc_timezone_offset()
    {
		$timezone = get_option( 'timezone_string' );

		if ( $timezone ) {
			$timezone_object = new DateTimeZone( $timezone );
			return $timezone_object->getOffset( new DateTime( 'now' ) );
		} else {
			return floatval( get_option( 'gmt_offset', 0 ) ) * HOUR_IN_SECONDS;
		}
	}

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/category/worker-elementor/';

    }

}
