<?php
namespace Merkulove\Selection;


use Elementor\Controls_Manager;
use Elementor\Controls_Stack;
use Merkulove\Selection\Unity\Plugin;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


final class ScrollingEffect
{

    /**
     * The one true ScrollingEffect.
     *
     * @since 1.0.0
     * @access private
     * @var ScrollingEffect
     **/
    private static $instance;


    public function __construct()
    {
        add_action( 'elementor/element/section/section_advanced/after_section_end', [ $this, 'register_controls' ], 10, 2 );

        add_action( 'elementor/element/column/section_advanced/after_section_end', [ $this, 'register_controls' ], 10, 2 );

        add_action( 'elementor/element/common/_section_style/after_section_end', [ $this, 'register_controls' ], 1 );


        add_action( 'elementor/frontend/after_enqueue_scripts', [ $this, 'enqueue_scripts' ] );

    }


    /**
     * Register scripts for scrolling effect.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void.
     */
    public function enqueue_scripts() {
        wp_enqueue_script(
            'mdp-selection-scrolling-effect',
               Plugin::get_url() . 'js/scrolling-effect' . Plugin::get_suffix() . '.js',
                   [ 'elementor-frontend' ], Plugin::get_version(),
           true
        );

    }


    /**
     * Generate controls for scrolling effect.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void.
     */
    private function generate_controls( $element, $controls_opts ) {
        $controls_data = [
            'heading_name' => array_key_exists( 'heading_name', $controls_opts ) ?
                $controls_opts['heading_name'] : '',
            'control_id_prefix' => array_key_exists( 'control_id_prefix', $controls_opts ) ?
                $controls_opts['control_id_prefix'] : '',
            'direction_options' => array_key_exists( 'direction_options', $controls_opts ) ?
                $controls_opts['direction_options'] : '',
            'anchors' => array_key_exists( 'anchors', $controls_opts ) ?
                $controls_opts['anchors'] : '',
            'speed_name' => array_key_exists( 'speed_name', $controls_opts ) ?
                $controls_opts['speed_name'] : 'Speed',
            'speed_min' => array_key_exists( 'speed_min', $controls_opts ) ?
                $controls_opts['speed_min'] : 1,
            'speed_max' => array_key_exists( 'speed_max', $controls_opts ) ?
                $controls_opts['speed_max'] : 10
        ];


        $element->add_control(
            $controls_data['control_id_prefix'] . '_enable',
            [
                'label'        => esc_html__( 'Enable ' . $controls_data['heading_name'], 'selection' ),
                'type'         => Controls_Manager::SWITCHER,
                'default'      => false,
                'return_value' => 'yes',
                'frontend_available' => true,
                'condition' => [
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->add_control(
            $controls_data['control_id_prefix'] . '_heading',
            [
                'label' => esc_html__( $controls_data['heading_name'], 'selection' ),
                'type' => Controls_Manager::HEADING,
                'condition' => [
                    $controls_data['control_id_prefix'] . '_enable' => 'yes',
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->add_control(
            $controls_data['control_id_prefix'] . '_direction',
            [
                'label' => esc_html__( 'Direction', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => array_keys( $controls_data['direction_options'] )[0],
                'options' => $controls_data['direction_options'],
                'frontend_available' => true,
                'condition' => [
                    $controls_data['control_id_prefix'] . '_enable' => 'yes',
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->add_control(
            $controls_data['control_id_prefix'] . '_speed',
            [
                'label' => esc_html__( $controls_data['speed_name'], 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'frontend_available' => true,
                'range' => [
                    'px' => [
                        'min' => $controls_data['speed_min'],
                        'max' => $controls_data['speed_max'],
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 5,
                ],
                'condition' => [
                    $controls_data['control_id_prefix'] . '_enable' => 'yes',
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->add_control(
            $controls_data['control_id_prefix'] . '_viewport',
            [
                'label' => esc_html__( 'Viewport', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'sizes' => [
                        'start' => 0,
                        'end' => 100,
                    ],
                    'unit' => '%',
                ],
                'labels' => [
                    esc_html__( 'Bottom', 'selection' ),
                    esc_html__( 'Top', 'selection' ),
                ],
                'scales' => 1,
                'handles' => 'range',
                'frontend_available' => true,
                'condition' => [
                    $controls_data['control_id_prefix'] . '_enable' => 'yes',
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

    }


    /**
     * Register controls for scrolling effect.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void.
     */
    public function register_controls( Controls_Stack $element ) {

        $element->start_controls_section(
            'mdp_selection_scrolling_effect',
            [
                'label' => esc_html__( 'Scrolling Effect', 'selection' ),
                'tab' => Controls_Manager::TAB_ADVANCED,
            ]
        );

        $element->add_control(
            'mdp_selection_scrolling_effect_enable',
            [
                'label'        => esc_html__( 'Enable', 'selection' ),
                'type'         => Controls_Manager::SWITCHER,
                'default'      => false,
                'return_value' => 'yes',
                'frontend_available' => true,
            ]
        );

        $this->generate_controls( $element, [
            'heading_name' => 'Vertical scroll',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_vertical',
            'direction_options' => [
                 'up' => esc_html__( 'Up', 'selection' ),
                 'down' => esc_html__( 'Down', 'selection' )
            ]
        ] );


        $this->generate_controls( $element, [
            'heading_name' => 'Horizontal scroll',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_horizontal',
            'direction_options' => [
                 'left' => esc_html__( 'Left', 'selection' ),
                 'right' => esc_html__( 'Right', 'selection' )
            ]
        ] );


        $this->generate_controls( $element, [
            'heading_name' => 'Transparency',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_transparency',
            'direction_options' => [
                'effect-in' => esc_html__( 'Fade in', 'selection' ),
                'effect-out' => esc_html__( 'Fade out', 'selection' ),
                'effect-out-in' => esc_html__( 'Fade out in', 'selection' ),
                'effect-in-out' => esc_html__( 'Fade in out', 'selection' )
            ],
            'speed_name' => 'Level'
        ] );


        $this->generate_controls( $element, [
            'heading_name' => 'Blur',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_blur',
            'direction_options' => [
                'effect-in' => esc_html__( 'Fade in', 'selection' ),
                'effect-out' => esc_html__( 'Fade out', 'selection' ),
                'effect-out-in' => esc_html__( 'Fade out in', 'selection' ),
                'effect-in-out' => esc_html__( 'Fade in out', 'selection' )
            ],
            'speed_name' => 'Level',
            'speed_min' => 1,
            'speed_max' => 15
        ] );

        $this->generate_controls( $element, [
            'heading_name' => 'Rotate',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_rotate',
            'direction_options' => [
                'left' => esc_html__( 'Left', 'selection' ),
                'right' => esc_html__( 'Right', 'selection' )
            ],
            'anchors' => true
        ] );

        $this->generate_controls( $element, [
            'heading_name' => 'Scale',
            'control_id_prefix' => 'mdp_selection_scrolling_effect_scale',
            'direction_options' => [
                'effect-in' => esc_html__( 'Scale up', 'selection' ),
                'effect-out' => esc_html__( 'Scale down', 'selection' ),
                'effect-out-in' => esc_html__( 'Scale down up', 'selection' ),
                'effect-in-out' => esc_html__( 'Scale up down', 'selection' )
            ],
            'speed_min' => -10,
            'speed_max' => 10,
            'anchors' => true
        ] );

        $element->add_control(
             'mdp_selection_scrolling_effect_x_anchor',
            [
                'label' => esc_html__( 'X anchor point', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-h-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-h-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-h-align-right',
                    ],
                ],
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'mdp_selection_scrolling_effect_enable',
                            'operator' => '==',
                            'value' => 'yes'
                        ],
                        [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'terms' => [
                                        [
                                            'name' => 'mdp_selection_scrolling_effect_scale_enable',
                                            'operator' => '==',
                                            'value' => 'yes'
                                        ],
                                    ]
                                ],
                                [
                                    'terms' => [
                                        [
                                            'name' => 'mdp_selection_scrolling_effect_rotate_enable',
                                            'operator' => '==',
                                            'value' => 'yes'
                                        ],
                                    ]
                                ]
                            ]
                        ]
                    ]
                ],
                'default' => 'center',
                'frontend_available' => true,
                'toggle' => true,
            ]
        );

        $element->add_control(
            'mdp_selection_scrolling_effect_y_anchor',
            [
                'label' => esc_html__( 'Y anchor point', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'top' => [
                        'title' => esc_html__( 'Top', 'selection' ),
                        'icon' => 'eicon-v-align-top',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-v-align-middle',
                    ],
                    'bottom' => [
                        'title' => esc_html__( 'Bottom', 'selection' ),
                        'icon' => 'eicon-v-align-bottom',
                    ],
                ],
                'conditions' => [
                    'relation' => 'and',
                    'terms' => [
                        [
                            'name' => 'mdp_selection_scrolling_effect_enable',
                            'operator' => '==',
                            'value' => 'yes'
                        ],
                        [
                            'relation' => 'or',
                            'terms' => [
                                [
                                    'terms' => [
                                        [
                                            'name' => 'mdp_selection_scrolling_effect_scale_enable',
                                            'operator' => '==',
                                            'value' => 'yes'
                                        ],
                                    ]
                                ],
                                [
                                    'terms' => [
                                        [
                                            'name' => 'mdp_selection_scrolling_effect_rotate_enable',
                                            'operator' => '==',
                                            'value' => 'yes'
                                        ],
                                    ]
                                ]
                            ]
                        ]
                    ]
                ],
                'default' => 'center',
                'frontend_available' => true,
                'toggle' => true,
            ]
        );

        $element->add_control(
            'mdp_selection_scrolling_effect_transition_duration',
            [
                'label' => esc_html__( 'Transition duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'frontend_available' => true,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 10,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1,
                ],
                'condition' => [
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->add_control(
            'mdp_selection_scrolling_effect_enable_on',
            [
                'label' => esc_html__( 'Enable On', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'label_block' => 'true',
                'default' => [ 'desktop', 'tablet', 'mobile' ],
                'options' => [
                    'desktop' => esc_html__( 'Desktop', 'selection' ),
                    'tablet' => esc_html__( 'Tablet', 'selection' ),
                    'mobile' => esc_html__( 'Mobile', 'selection' ),
                ],
                'frontend_available' => true,
                'condition' => [
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ],
            ]
        );

        $element->add_control(
            'mdp_selection_scrolling_effect_relative_to',
            [
                'label' => esc_html__( 'Effect relative to', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'viewport',
                'options' => [
                    'viewport'  => esc_html__( 'Viewport', 'selection' ),
                    'entire-page' => esc_html__( 'Entire page', 'selection' ),
                ],
                'condition' => [
                    'mdp_selection_scrolling_effect_enable' => 'yes'
                ]
            ]
        );

        $element->end_controls_section();

    }


    /**
     * Main ScrollingEffect Instance.
     * Insures that only one instance of ScrollingEffect exists in memory at any one time.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return ScrollingEffect
     **/
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

            self::$instance = new self;

        }

        return self::$instance;

    }


}