<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use Elementor\Plugin;
use WP_Query;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Templates class contain Selection Templates Logic
 * @since 1.0.0
 **/
final class Templates {

	/**
	 * The one true Templates.
	 *
     * @since 1.0.0
     * @access private
	 * @var Templates
	 **/
	private static $instance;

	/** @var array[] Selection Template Types  */
	public static $selection_tmpl = array(
		'header' => array(),
		'footer' => array(),
		'before' => array(),
		'after' => array(),
		'error_404' => array()
	);

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function __construct() {

	    /** Prepare Selection templates */
	    add_action( 'wp', [ $this, 'set_selection_tmpl_section' ] );
	    add_action( 'wp', [ $this, 'set_selection_tmpl_page' ] );

	    /** Show elementor headers */
	    add_action( 'wp_body_open', [ $this, 'display_selection_headers' ] );

	    /** Show elementor footer */
	    add_action( 'wp_footer', [ $this, 'display_selection_footer' ] );

	    /** Display elementor templates before and after post content */
	    add_filter( 'the_content', [ $this, 'display_selection_content' ] );

    }

	/**
	 * Display headers template after body open
	 */
    public function display_selection_headers() {

	    foreach ( self::$selection_tmpl[ 'header' ] as $header_id ) {

		    echo $this->get_elementor_content( $header_id );

	    }

	    if ( is_404() ) {

		    foreach ( self::$selection_tmpl[ 'error_404' ] as $error_page_id ) {

			    echo $this->get_elementor_content( $error_page_id );
			    get_footer();
			    exit;

		    }

	    }

    }

	/**
	 * Display headers template after body open
	 */
	public function display_selection_footer() {

		foreach ( self::$selection_tmpl[ 'footer' ] as $footer_id ) {

			echo $this->get_elementor_content( $footer_id );

		}

	}

	/**
	 * Add Elementor template before or after post content
	 *
	 * @param $content
	 *
	 * @return mixed|string
	 */
	public function display_selection_content( $content ) {

		// Add template before content
		foreach ( self::$selection_tmpl[ 'before' ] as $before_id ) {

			$content = $this->get_elementor_content( $before_id ) . $content;

		}

		// Add template after content
		foreach ( self::$selection_tmpl[ 'after' ] as $after_id ) {

			$content .= $this->get_elementor_content( $after_id );

		}

		return $content;

	}

	/**
	 * Find Selection Section templates and store it
	 */
    public function set_selection_tmpl_section() {

    	// Get id of the displayed post
	    $current_post_id = get_the_ID();
	    if ( ! $current_post_id ) { return; }

	    // Collect templates id by template type
	    foreach ( self::$selection_tmpl as $tmpl_type => $templates ) {

	    	// Get templates by type
		    $selection_tmpl_typed = $this->get_selection_templates( $tmpl_type );

		    // Collect templates id of the single type
		    foreach ( $selection_tmpl_typed as $single_tmpl ) {

			    // Dont show template itself
			    if ( $single_tmpl->ID === $current_post_id ) { continue; }

			    // Show only if assignments returns true
			    $assignments = get_post_meta( $single_tmpl->ID, 'mdp_selection_assignments_settings', true );
			    if ( ! AssignmentsMetabox::get_instance()->display( $assignments ) ) { continue; }

			    // Store id
			    self::$selection_tmpl[ $tmpl_type ][ $single_tmpl->ID ] = $single_tmpl->ID;

		    }

	    }

    }

	/**
	 * Find Selection Page Templates and store it
	 */
	public function set_selection_tmpl_page() {

		// Error 404 - Page Template */
		if ( is_404() ) {

			$tmpl_type = 'error_404';

			// Get templates by type
			$selection_tmpl_typed = $this->get_selection_templates( $tmpl_type );

			// Collect templates id of the single type
			foreach ( $selection_tmpl_typed as $single_tmpl ) {

				// Show only if assignments returns true
				$assignments = get_post_meta( $single_tmpl->ID, 'mdp_selection_assignments_settings', true );
				if ( ! AssignmentsMetabox::get_instance()->display( $assignments ) ) { continue; }

				// Store id
				self::$selection_tmpl[ $tmpl_type ][ $single_tmpl->ID ] = $single_tmpl->ID;

			}

		}

	}

	/**
	 * Returns WP_post data of single Elementor Template
	 * @param string $position
	 *
	 * @return array
	 */
	private function get_selection_templates( $position = 'header' ) {

		$args = array(
			'post_type' => TemplatesMetaboxes::POST_TYPE,
			'post_status' => 'publish',
			'posts_per_page' => -1,
			'meta_query' => [
				[
					'key' => 'mdp_selection_position',
					'value' => $position,
					'compare' => '=',
				]
			],

		);

		/** Create Instance. */
		$messages_query = new WP_Query;

		/** Run Query. */
		return $messages_query->query( $args );

	}

	/**
	 * Returns Elementor post content
	 *
	 * @param $id - post ID
	 *
	 * @return string - Content
	 */
	private function get_elementor_content( $id ) {

		return class_exists( '\Elementor\Plugin' ) ?
			Plugin::$instance->frontend->get_builder_content_for_display( $id ) :
			'';

	}

	/**
	 * Main Templates Instance.
	 * Insures that only one instance of Templates exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Templates
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
