<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use WP_Post;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class manage Selection meta-boxes
 * @since 1.0.0
 **/
final class TemplatesMetaboxes {

	/**
	 * Elementor Template Post Type name.
	 *
	 * @since 1.0.0
	 * @var string
	 **/
	const POST_TYPE = 'elementor_library';

	/**
	 * The one true TemplatesMetaboxes.
	 *
	 * @var TemplatesMetaboxes
	 * @since 1.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Add Selection column to Elementor Templates Library list */
		add_filter( 'manage_' . self::POST_TYPE . '_posts_columns', [ $this, 'add_head_selection_column' ], 10 );
		add_action( 'manage_' . self::POST_TYPE . '_posts_custom_column', [ $this, 'add_content_selection_column' ], 10, 2 );

		/** Fire meta box setup on the post editor screen. */
		add_action( 'load-post.php', [ $this, 'meta_boxes_setup' ] );
		add_action( 'load-post-new.php', [ $this, 'meta_boxes_setup' ] );

	}

	/**
	 * Save meta box values.
	 *
	 * @param $post_id
	 * @param $post
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void|mixed
	 **/
	public function save_meta_boxes( $post_id, $post ) {

		/** Verify the nonce before proceeding. */
		if ( ! isset( $_POST['metabox_fields_nonce'] ) || ! wp_verify_nonce( $_POST['metabox_fields_nonce'], basename( __FILE__ ) ) ) {
			return $post_id;
		}

		/** Get the post type object. */
		$post_type = get_post_type_object( $post->post_type );
		if ( null === $post_type ) { return $post_id; }

		/** Check if the current user has permission to edit the post. */
		if ( ! current_user_can( $post_type->cap->edit_post, $post_id ) ) {
			return $post_id;
		}

		/** Keys of fields to save. */
		$fields = [

			'mdp_selection_assignments_settings',
            'mdp_selection_position'

        ];

		/** Update each field. */
		foreach ( $fields as $f_key ) {

		    $field = isset( $_POST[$f_key] ) ? $_POST[$f_key] : '';
            update_post_meta( $post_id, $f_key, sanitize_text_field( $field ) );

        }

		/** Add post id as meta-value */
		update_post_meta( $post_id, 'selection_post_id', $post_id );

	}

	/**
	 * Create meta boxes to be displayed on Message editor screen.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function add_meta_boxes() {

        /** Add meta box to Elementor Templates */

		add_meta_box(
            'mdp-selection-conditions',
            esc_html__( 'Display condition', 'selection' ),
            [ $this, 'meta_box_assignments' ],
			self::POST_TYPE,
            'normal',
            'high'
        );

		add_meta_box(
			'mdp-selection-position',
			esc_html__( 'Position', 'selection' ),
			[ $this, 'meta_box_position' ],
			self::POST_TYPE,
			'side',
			'core'
		);

	}

	/**
	 * @param WP_Post $message
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function meta_box_assignments( WP_Post $message ) {

		/** Nonce field to validate form request came from current site. */
		wp_nonce_field( basename( __FILE__ ), 'metabox_fields_nonce' );

		AssignmentsMetabox::get_instance()->render_assignments( $message );

	}

	/**
	 * @param WP_Post $message
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function meta_box_position( WP_Post $message ) {

		/** Nonce field to validate form request came from current site. */
		wp_nonce_field( basename( __FILE__ ), 'metabox_fields_nonce' );

		$this->render_position_fields( $message );

	}

	/**
	 * Popup style controls
	 * @param $message
	 */
	private function render_position_fields( $message ) {

		$positions = [
			'none'      => esc_html__( 'Unset', 'selection' ),
			'header'    => esc_html__( 'Header', 'selection' ),
			'footer'    => esc_html__( 'Footer', 'selection' ),
			'before'    => esc_html__( 'Before Post Content', 'selection' ),
			'after'     => esc_html__( 'After Post Content', 'selection' ),
			'error_404'       => esc_html__( '404 Error Page', 'selection' )
		];

		MetaBoxUI::get_instance()->render_select(
			$message,
			'mdp_selection_position',
			$positions,
			is_array( $positions ) ? reset( $positions ) : '',// First in array is default,
			esc_html__( 'Position:', 'selection' ),
			esc_html__( 'Select the template position', 'selection' )
		);

	}

	/**
	 * Meta box setup function.
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function meta_boxes_setup() {

		/** Add meta boxes on the 'add_meta_boxes' hook. */
		add_action( 'add_meta_boxes', [ $this, 'add_meta_boxes' ] );

		/** Save Left and Right values on the 'save_post' hook. */
		add_action( 'save_post', [ $this, 'save_meta_boxes' ], 1, 2 );

	}

	/**
	 * Add HEAD for custom column
	 *
	 * @param array $columns
	 *
	 * @since  1.0.0
	 * @access public
	 * @return array
	 */
	public function add_head_selection_column( $columns ) {

	    /** @var string Column heading title $column_selection */
        $column_selection = [ 'selection_use' => esc_html__( 'Selection', 'selection' ) ];

		/** Set selection column to the penultimate position */
		return array_merge( array_splice( $columns, 0, -1 ), $column_selection, $columns );

	}

	/**
	 * Add CONTENT for custom column
	 *
	 * @param $column_name
	 * @param $post_ID
	 *
	 * @since  1.0.0
	 * @access public
	 **/
	public function add_content_selection_column( $column_name, $post_ID ) {

		if ( $column_name !== 'selection_use' ) { return; }

		// Get position from meta-boxes data
		$position = get_post_meta( $post_ID, 'mdp_selection_position', true );
		if ( ! $position || $position === 'none' ) { return; }

		// Get assignments
		$default_assignments = 'WPContent|:0,|WPContentVal|:||,|homePage|:0,|menuItems|:0,|menuItemsVal|:||,|dateTime|:0,|dateTimeStart|:||,|dateTimeEnd|:||,|userRoles|:0,|userRolesVal|:||,|URL|:0,|URLVal|:||,|devices|:0,|devicesVal|:||,|PHP|:0';
		$assignments = get_post_meta( $post_ID, 'mdp_selection_assignments_settings', true );

		// Print the template position
		if ( ! strpos( $assignments, $default_assignments ) ) {
			esc_html_e( ucwords( $position ), 'selection' );
		} else {
			esc_html_e( 'Global ' . $position, 'selection' );
		}

	}

	/**
	 * Main TemplatesMetaboxes Instance.
	 *
	 * Insures that only one instance of TemplatesMetaboxes exists in memory at any one time.
	 *
	 * @static
	 * @return TemplatesMetaboxes
	 * @since 1.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;
		
	}

}
